@@ -1,13 +1,15 @@
 =begin
   top-level extconf.rb for Ruby-GNOME2
 
-  $Id: extconf.rb,v 1.15 2006/12/26 15:49:16 mutoh Exp $
+  $Id: extconf.rb,v 1.17 2007/10/22 12:19:17 ktou Exp $
 
   Copyright (C) 2003-2005 Ruby-GNOME2 Project Team
 =end
 
+require 'English'
 require 'mkmf'
-require 'ftools'
+require 'fileutils'
+require 'pathname'
 
 priorlibs = ["glib", "gdkpixbuf", "pango", "atk", "gtk"]
 
@@ -17,10 +19,9 @@ priorlibs = ["glib", "gdkpixbuf", "pango", "atk", "gtk"]
 $ruby = File.join(Config::CONFIG['bindir'], Config::CONFIG['RUBY_INSTALL_NAME'] + Config::CONFIG['EXEEXT'])
 $ruby = arg_config("--ruby", $ruby)
 
-rm = "rm -f "
+rm = "rm -f"
 if /mswin32/ =~ RUBY_PLATFORM
-  rm = "del " 
-  $ruby.gsub!(/\//, '\\')
+  rm = "del"
 end
 
 
@@ -29,6 +30,10 @@ $topsrcdir = $configure_args["--topsrcdir"] ||= $srcdir
 $topdir = $configure_args["--topdir"] ||= Dir.pwd
 $strict = $configure_args["--strict"] ? "--strict" : ""
 
+$srcdir = File.expand_path($srcdir)
+$topsrcdir = File.expand_path($topsrcdir)
+$topdir = File.expand_path($topdir)
+
 subdirs = ARGV.select{|v|  /^--/ !~ v}
 
 if subdirs.size == 0
@@ -47,18 +52,25 @@ end
 #
 targets = []
 ignore = []
+
+ruby, *ruby_args = Shellwords.shellwords($ruby)
+if ARGV.grep(/\A--ruby=/)
+  extra_args = ["--ruby=#{$ruby}"] + ARGV.reject {|arg| /\A--ruby=/ =~ arg}
+else
+  extra_args = ARGV.dup
+end
+
 subdirs.each do |subdir|
-  if /mingw/ =~ RUBY_PLATFORM
-    $ruby.gsub!('\\', '/')
-  end
   STDERR.puts("#{$0}: Entering directory `#{subdir}'")
-  File.mkpath(subdir)
+  FileUtils.mkdir_p(subdir)
   topdir = File.join(*([".."] * subdir.split(/\/+/).size))
-  /^\// =~ (dir = $topsrcdir) or dir = File.join(topdir, $topsrcdir)
+  dir = $topsrcdir
+  dir = File.join(topdir, dir) unless Pathname.new(dir).absolute?
   srcdir = File.join(dir, subdir)
-  ret = system($ruby, "-C", subdir, File.join(srcdir, "extconf.rb"),
-   "--topsrcdir=#{dir}", "--topdir=#{topdir}", "--srcdir=#{srcdir}", "--ruby=#{$ruby}",
-   *ARGV)
+  args = ruby_args + ["-C", subdir, File.join(srcdir, "extconf.rb"),
+                      "--topsrcdir=#{dir}", "--topdir=#{topdir}",
+                      "--srcdir=#{srcdir}", *extra_args]
+  ret = system(ruby, *args)
   STDERR.puts("#{$0}: Leaving directory '#{subdir}'")
   if ret
     targets << subdir
@@ -74,33 +86,63 @@ puts "Ignored libraries: #{ignore.join(', ')}" if ignore.size > 0
 # generate top-level Makefile
 #
 
-if /mingw/ =~ RUBY_PLATFORM
-  $ruby.gsub!('\\', '/')
+def run_make_in_sub_dirs(command)
+  if /mswin32/ =~ RUBY_PLATFORM
+    "	$(COMMAND) '$(SUBDIRS)' $(MAKE) #{command}"
+  else
+    <<-EOS.chomp
+	@(					\\
+	  succeeded='';				\\
+	  failed='';				\\
+	  for dir in $(SUBDIRS); do		\\
+	    (cd $$dir; $(MAKE) #{command});	\\
+	    if [ $$? -eq 0 ]; then		\\
+	      succeeded="$$succeeded $$dir";	\\
+	    else				\\
+	      failed="$$failed $$dir";		\\
+	    fi;					\\
+	  done;					\\
+	  if [ "$$succeeded" = "" ]; then	\\
+	    succeeded="NONE";			\\
+	  fi;					\\
+	  if [ "$$failed" = "" ]; then		\\
+	    failed="NONE";			\\
+	  fi;					\\
+	  echo;					\\
+	  echo "-----";				\\
+	  echo "SUCCEEDED: $$succeeded";	\\
+	  echo "FAILED: $$failed";		\\
+	  echo "-----";				\\
+	  echo "Done.";				\\
+	)
+    EOS
+  end
 end
 
+
 File.open("Makefile", "w") do |makefile|
-  makefile.print("\
+  makefile.print(<<-EOM)
 TOPSRCDIR = #{$topsrcdir}
 SUBDIRS = #{targets.join(' ')}
 COMMAND = #{$ruby} #{$topsrcdir}/exec_make.rb #{$strict}
 RM = #{rm}
 
 all:
-	$(COMMAND) '$(SUBDIRS)' $(MAKE) all
+#{run_make_in_sub_dirs('all')}
 
 install:
-	$(COMMAND) '$(SUBDIRS)' $(MAKE) install
+#{run_make_in_sub_dirs('install')}
 
 site-install:
-	$(COMMAND) '$(SUBDIRS)' $(MAKE) site-install
+#{run_make_in_sub_dirs('site-install')}
 
 clean:
-	$(COMMAND) '$(SUBDIRS)' $(MAKE) clean
+#{run_make_in_sub_dirs('clean')}
 
 distclean:
-	$(COMMAND) '$(SUBDIRS)' $(MAKE) distclean
+#{run_make_in_sub_dirs('distclean')}
 	$(RM) Makefile mkmf.log
-")
+EOM
 end
 
 puts "-----"
