@@ -11,8 +11,9 @@
 
 require 'mkmf'
 require 'pkg-config'
+require 'glib-mkenums'
 
-unless defined? macro_defined?
+unless defined?(macro_defined?)
   def macro_defined?(macro, src, opt="")
     try_cpp(src + <<EOP, opt)
 #ifndef #{macro}
@@ -27,7 +28,7 @@ STDOUT.print("checking for GCC... ")
 STDOUT.flush
 if macro_defined?("__GNUC__", "")
   STDOUT.print "yes\n"
-  $CFLAGS += ' -Wall' 
+  $CFLAGS += ' -Wall'
   $cc_is_gcc = true
 else
   STDOUT.print "no\n"
@@ -38,7 +39,7 @@ include_path = nil
 if ENV['GTK_BASEPATH'] and /cygwin/ !~ RUBY_PLATFORM
   include_path = (ENV['GTK_BASEPATH'] + "\\INCLUDE").gsub("\\", "/")
 #  $hdrdir += " -I#{include_path} "
-  $CFLAGS += " -I#{include_path} " 
+  $CFLAGS += " -I#{include_path} "
 end
 
 def check_win32()
@@ -111,85 +112,120 @@ def setup_win32(target_name)
 end
 
 #add_depend_package("glib2", "glib/src", "/...../ruby-gnome2")
-def add_depend_package(target_name, target_srcdir, topdir)
-  $CFLAGS += " -I" + File.join(topdir, target_srcdir)
+def add_depend_package(target_name, target_srcdir, top_srcdir)
+  [top_srcdir, $configure_args['--topdir']].each do |topdir|
+    topdir = File.expand_path(topdir)
+    if RUBY_VERSION < "1.8.5"
+      $CFLAGS = "-I#{File.join(topdir, target_srcdir)} #{$CFLAGS}"
+    else
+      $INCFLAGS = "-I#{File.join(topdir, target_srcdir)} #{$INCFLAGS}"
+    end
 
-  if /cygwin|mingw/ =~ RUBY_PLATFORM
-    $libs << " -lruby-#{target_name}"
-    $LDFLAGS << " -L#{topdir}/#{target_srcdir}"
-  elsif /mswin32/ =~ RUBY_PLATFORM
-    $DLDFLAGS << " /libpath:#{topdir}/#{target_srcdir}"
-    $libs << " libruby-#{target_name}.lib"
+    if /cygwin|mingw/ =~ RUBY_PLATFORM
+      $libs << " -lruby-#{target_name}"
+      $LDFLAGS << " -L#{topdir}/#{target_srcdir}"
+    elsif /mswin32/ =~ RUBY_PLATFORM
+      $DLDFLAGS << " /libpath:#{topdir}/#{target_srcdir}"
+      $libs << " libruby-#{target_name}.lib"
+    end
   end
 end
 
 def add_distcleanfile(file)
-  $distcleanfiles = [] unless $distcleanfiles 
+  $distcleanfiles ||= []
   $distcleanfiles << file
 end
 
 def create_makefile_at_srcdir(pkg_name, srcdir, defs = nil)
-  begin
-    Dir.mkdir(srcdir) unless File.exist? srcdir
-    Dir.chdir srcdir
+  base_dir = File.basename(Dir.pwd)
+  last_common_index = srcdir.rindex(base_dir)
+  if last_common_index
+    builddir = srcdir[(last_common_index + base_dir.size + 1)..-1]
+  end
+  builddir ||= "."
+  FileUtils.mkdir_p(builddir)
 
+  Dir.chdir(builddir) do
     yield if block_given?
 
     $defs << defs if defs
+
+    pattern = "*.{#{SRC_EXT.join(',')}}"
+    srcs = Dir[File.join(srcdir, pattern)]
+    srcs |= Dir[File.join(".", pattern)]
+    $objs = srcs.collect do |src|
+      File.basename(src, ".*") + ".o"
+    end.flatten.uniq
+
     create_makefile(pkg_name, srcdir)
-  ensure
-    Dir.chdir('..')
   end
 end
 
-def create_top_makefile(sub_dirs = ["src"])
-  mfile = File.open("Makefile", "w")
+def run_make_in_sub_dirs_command(command, sub_dirs)
   if /mswin32/ =~ RUBY_PLATFORM
-    mfile.print <<MSWIN32_END 
+    sub_dirs.collect do |dir|
+      <<-EOM.chmop
+	@cd #{dir}
+	@nmake -nologo DESTDIR=$(DESTDIR) #{command}
+	@cd ..
+      EOM
+    end.join("\n")
+  else
+    sub_dirs.collect do |dir|
+      "\t@cd #{dir}; $(MAKE) #{command}"
+    end.join("\n")
+  end
+end
 
+def create_top_makefile(sub_dirs=["src"])
+  File.open("Makefile", "w") do |makefile|
+    makefile.print(<<-EOM)
 all:
-#{sub_dirs.map{|d| "	@cd #{d}\n	@nmake -nologo all\n	@cd ..\n"}.join('')}
+#{run_make_in_sub_dirs_command("all", sub_dirs)}
+
 install:
-#{sub_dirs.map{|d| "	@cd #{d}\n	@nmake -nologo install DESTDIR=$(DESTDIR)\n	@cd ..\n"}.join('')}
+#{run_make_in_sub_dirs_command("install", sub_dirs)}
+
 site-install:
-#{sub_dirs.map{|d| "	@cd #{d}\n	@nmake -nologo site-install DESTDIR=$(DESTDIR)\n	@cd ..\n"}.join('')}
+#{run_make_in_sub_dirs_command("site-install", sub_dirs)}
+
 clean:
-#{sub_dirs.map{|d| "	@cd #{d}\n	@nmake -nologo clean\n	@cd ..\n"}.join('')}	@if exist extconf.h del extconf.h
+#{run_make_in_sub_dirs_command("clean", sub_dirs)}
+    EOM
+
+    if /mswin32/ =~ RUBY_PLATFORM
+      makefile.print(<<-EOM)
+	@if exist extconf.h del extconf.h
 	@if exist conftest.* del conftest.*
 	@if exist *.lib del *.lib
 	@if exist *~ del *~
-	@if exist mkmf.log del mkmf.log 
-MSWIN32_END
-  else
-    mfile.print <<END
-all:
-#{sub_dirs.map{|d| "	@cd #{d}; make all\n"}.join('')}
+	@if exist mkmf.log del mkmf.log
+      EOM
+    else
+      makefile.print(<<-EOM)
 
-install:
-#{sub_dirs.map{|d| "	@cd #{d}; make install\n"}.join('')}
-site-install:
-#{sub_dirs.map{|d| "	@cd #{d}; make site-install\n"}.join('')}
-clean:
-#{sub_dirs.map{|d| "	@cd #{d}; make clean\n"}.join('')}
-distclean:	clean
-#{sub_dirs.map{|d| "	@cd #{d}; make distclean\n"}.join('')}
+distclean: clean
+#{run_make_in_sub_dirs_command("distclean", sub_dirs)}
 	@rm -f Makefile extconf.h conftest.*
 	@rm -f core *~ mkmf.log
-END
+      EOM
+    end
   end
-  mfile.close
 end
 
 # This is used for the library which doesn't support version info.
 def make_version_header(app_name, pkgname, dir = "src")
   version = PKGConfig.modversion(pkgname).split(/\./)
-
+  (0..2).each do |v|
+    version[v] = "0" unless version[v]
+  end
   filename = "rb#{app_name.downcase}version.h"
 
   puts "creating #{filename}"
 
   add_distcleanfile(filename)
 
+  FileUtils.mkdir_p(dir)
   out = File.open(File.join(dir, filename), "w")
 
   out.print %Q[/* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */
@@ -224,8 +260,9 @@ def check_ruby_func
   $CPPFLAGS << " -I$(sitearchdir) "
 
   #Other options
-  have_func("rb_define_alloc_func") # for ruby-1.8
-  have_func("rb_block_proc") # for ruby-1.8
+  ruby_header = "ruby.h"
+  have_func("rb_define_alloc_func", ruby_header) # for ruby-1.8
+  have_func("rb_block_proc", ruby_header) # for ruby-1.8
 
   STDOUT.print("checking for new allocation framework... ") # for ruby-1.7
   if Object.respond_to? :allocate
@@ -253,5 +290,50 @@ def add_obj(name)
   $objs << name unless $objs.index(name)
 end
 
+def glib_mkenums(prefix, files, g_type_prefix, include_files)
+  add_distcleanfile(prefix + ".h")
+  add_distcleanfile(prefix + ".c")
+  GLib::MkEnums.create(prefix, files, g_type_prefix, include_files)
+end
+
+def check_cairo
+  return false unless PKGConfig.have_package('cairo')
+
+  have_rb_cairo_h = have_header('rb_cairo.h')
+  unless have_rb_cairo_h
+    begin
+      require 'rubygems'
+      gem 'cairo'
+      require 'cairo'
+      rcairo_src_gem_path_re =
+        /\A#{Regexp.escape(Gem.dir)}\/gems\/cairo-[\d.]+\/src\z/
+      $:.each do |path|
+        if rcairo_src_gem_path_re =~ path
+          $CFLAGS += " -I#{path} "
+          have_rb_cairo_h = have_header('rb_cairo.h')
+          break
+        end
+      end
+    rescue LoadError
+    end
+  end
+
+  if have_rb_cairo_h
+    if /mingw|cygwin|mswin32/ =~ RUBY_PLATFORM
+      unless ENV["CAIRO_PATH"]
+        puts "Error! Set CAIRO_PATH."
+        exit 1
+      end
+      add_depend_package("cairo", "src", ENV["CAIRO_PATH"])
+      $defs << "-DRUBY_CAIRO_PLATFORM_WIN32"
+    end
+  end
+  have_rb_cairo_h
+end
 
 check_ruby_func
+
+if /mingw/ =~ RUBY_PLATFORM
+  $ruby.gsub!('\\', '/')
+end
+
