@@ -3,8 +3,8 @@
 
   rbglib.h -
 
-  $Author: mutoh $
-  $Date: 2006/12/29 08:43:24 $
+  $Author: sakai $
+  $Date: 2007/07/04 13:13:20 $
 
   Copyright (C) 2002-2005  Ruby-GNOME2 Project
   Copyright (C) 2002,2003  Masahiro Sakai
@@ -22,39 +22,40 @@ extern "C" {
 #endif /* __cplusplus */
 
 #define RBGLIB_MAJOR_VERSION 0
-#define RBGLIB_MINOR_VERSION 16
+#define RBGLIB_MINOR_VERSION 19
 #define RBGLIB_MICRO_VERSION 0
 
-#ifndef StringValue
-#define StringValue(s) ((s) = (TYPE(s) == T_STRING) ? (s) : rb_str_to_str(s))
-#endif
-#ifndef SafeStringValue
-#define SafeStringValue(v) rb_check_safe_str(StringValue(v))
-#endif
-#ifndef StringValuePtr
-#define StringValuePtr(v) rbg_string_value_ptr(&(v))
+#ifndef RSTRING_PTR
+#  define RSTRING_PTR(s) (RSTRING(s)->ptr)
+#  define RSTRING_LEN(s) (RSTRING(s)->len)
 #endif
 
-#ifndef LONG2NUM
-#define LONG2NUM INT2NUM
+#ifndef RARRAY_PTR
+#  define RARRAY_PTR(s) (RARRAY(s)->ptr)
+#  define RARRAY_LEN(s) (RARRAY(s)->len)
 #endif
 
-#ifndef ULONG2NUM
-#define ULONG2NUM UINT2NUM
+#ifndef DBL2NUM
+#  define DBL2NUM(v)      (rb_float_new(v))
 #endif
 
 #if ! GLIB_CHECK_VERSION(2,4,0)
 typedef int GPid;
 #endif
 
+#define RBG_INSPECT(object) (rbg_rval_inspect(object))
+
 #define RVAL2CSTR(v) (StringValuePtr(v))
-#define RVAL2CSTR2(v) (NIL_P(v) ? NULL : StringValuePtr(v))
+#define RVAL2CSTR_ACCEPT_NIL(v) (rbg_rval2cstr_accept_nil(v))
+#define RVAL2CSTR2(v) (RVAL2CSTR_ACCEPT_NIL(v))
 #define CSTR2RVAL(s) (rbg_cstr2rval(s))
-#define CSTR2RVAL2(s) (rbg_cstr2rval_with_free(s))
+#define CSTR2RVAL_FREE(s) (rbg_cstr2rval_with_free(s))
+#define CSTR2RVAL2(s) (CSTR2RVAL_FREE(s))
 
 #define CBOOL2RVAL(b)   ((b) ? Qtrue : Qfalse)
 #define RVAL2CBOOL(b)   (RTEST(b))
-#define RAISE_GERROR(error) rb_exc_raise(rbgerr_gerror2exception(error))
+#define GERROR2RVAL(error) (rbgerr_gerror2exception(error))
+#define RAISE_GERROR(error) rb_exc_raise(GERROR2RVAL(error))
 #define G_DEF_ERROR(domain, name, module, parent, gtype)         \
     rbgerr_define_gerror(domain, name, module, parent, gtype)
 #define G_DEF_ERROR2(domain, name, module, parent) \
@@ -71,13 +72,17 @@ typedef int GPid;
 #endif
 
 RUBY_GLIB2_VAR VALUE mGLib;
-extern gchar* rbg_string_value_ptr(volatile VALUE* ptr);
+
+extern const gchar *rbg_rval_inspect(VALUE object);
+
+extern gchar* rbg_string_value_ptr(volatile VALUE* ptr); /* no longer used */
+extern gchar *rbg_rval2cstr_accept_nil(VALUE str);
 extern VALUE rbg_cstr2rval(const char* str);
 extern VALUE rbg_cstr2rval_with_free(gchar* str);
 
 /* rbgerror.h */
 extern VALUE rbgerr_gerror2exception(GError *error);
-extern VALUE rbgerr_define_gerror(GQuark domain, gchar* name, VALUE module, VALUE parent, VALUE gtype);
+extern VALUE rbgerr_define_gerror(GQuark domain, const gchar* name, VALUE module, VALUE parent, VALUE gtype);
 
 extern VALUE rbglib_int64_to_num(guint64 val);
 extern VALUE rbglib_uint64_to_num(guint64 val);
