@@ -3,13 +3,21 @@
 
   rbglib_maincontext.c -
 
-  $Author: mutoh $
-  $Date: 2006/12/10 17:24:22 $
+  $Author: ktou $
+  $Date: 2007/08/30 10:13:44 $
 
   Copyright (C) 2005 Masao Mutoh
 ************************************************/
 
-#include "global.h"
+#include "rbgprivate.h"
+
+#ifndef HAVE_RB_THREAD_BLOCKING_REGION
+#  include <rubysig.h>
+#  include <node.h>
+#  ifdef HAVE_CURR_THREAD
+#    define rb_curr_thread curr_thread
+#  endif
+#endif
 
 /*
 static ID id_poll_func;
@@ -27,6 +35,305 @@ typedef struct _callback_info_t
     guint id;
 } callback_info_t;
 
+/*****************************************/
+static GPollFunc default_poll_func;
+
+#ifdef HAVE_RB_THREAD_BLOCKING_REGION
+
+/* just for ruby-1.9.0. */
+#if !defined(RUBY_UBF_IO) && defined(RB_UBF_DFL)
+#  define RUBY_UBF_IO RB_UBF_DFL
+#endif
+
+typedef struct _PollInfo
+{
+    GPollFD *ufds;
+    guint nfsd;
+    gint timeout;
+    gint result;
+} PollInfo;
+
+static VALUE
+rg_poll_in_blocking(void *data)
+{
+    PollInfo *info = data;
+
+    info->result = default_poll_func(info->ufds, info->nfsd, info->timeout);
+
+    return Qnil;
+}
+
+static gint
+rg_poll(GPollFD *ufds, guint nfsd, gint timeout)
+{
+    PollInfo info;
+
+    info.ufds = ufds;
+    info.nfsd = nfsd;
+    info.timeout = timeout;
+    info.result = 0;
+
+    rb_thread_blocking_region(rg_poll_in_blocking, &info, RUBY_UBF_IO, NULL);
+
+    return info.result;
+}
+#else
+static gint
+rg_poll(GPollFD *ufds, guint nfsd, gint timeout)
+{
+    gint result;
+
+    TRAP_BEG;
+    result = default_poll_func(ufds, nfsd, timeout);
+    TRAP_END;
+
+    return result;
+}
+#endif
+
+static void
+restore_poll_func(VALUE data)
+{
+    if (g_main_context_get_poll_func(NULL) == (GPollFunc)rg_poll) {
+        g_main_context_set_poll_func(NULL, default_poll_func);
+    }
+}
+
+#ifndef HAVE_RB_THREAD_BLOCKING_REGION
+/* from eval.c */
+#define WAIT_FD		(1<<0)
+#define WAIT_SELECT	(1<<1)
+#define WAIT_TIME	(1<<2)
+#define WAIT_JOIN	(1<<3)
+#define WAIT_PID	(1<<4)
+
+#define DELAY_INFTY 1E30
+
+static double
+timeofday()
+{
+    struct timeval tv;
+    gettimeofday(&tv, NULL);
+    return (double)tv.tv_sec + (double)tv.tv_usec * 1e-6;
+}
+
+/*****************************************/
+typedef struct _RGSource
+{
+    GSource source;
+
+    GList *poll_fds;
+    GList *old_poll_fds;
+    gboolean ready;
+} RGSource;
+
+static void
+source_cleanup_poll_fds(GSource *source)
+{
+    RGSource *rg_source = (RGSource *)source;
+    GList *node;
+
+    for (node = rg_source->old_poll_fds; node; node = g_list_next(node)) {
+        GPollFD *poll_fd = node->data;
+
+        g_source_remove_poll(source, poll_fd);
+        g_slice_free(GPollFD, poll_fd);
+    }
+    g_list_free(rg_source->old_poll_fds);
+    rg_source->old_poll_fds = NULL;
+}
+
+static inline void
+source_prepare_add_poll_fd(GSource *source, gint fd, guchar events)
+{
+    GPollFD *poll_fd;
+    GList *node;
+    RGSource *rg_source = (RGSource *)source;
+
+    for (node = rg_source->old_poll_fds; node; node = g_list_next(node)) {
+        poll_fd = node->data;
+        if (poll_fd->fd == fd && poll_fd->events == events) {
+            rg_source->old_poll_fds =
+                g_list_remove_link(rg_source->old_poll_fds, node);
+            rg_source->poll_fds = g_list_concat(rg_source->poll_fds, node);
+            return;
+        }
+    }
+
+    poll_fd = g_slice_new0(GPollFD);
+    poll_fd->fd = fd;
+    poll_fd->events = events;
+
+    g_source_add_poll(source, poll_fd);
+    rg_source->poll_fds = g_list_prepend(rg_source->poll_fds, poll_fd);
+}
+
+static inline void
+source_prepare_add_poll(GSource *source, rb_thread_t thread)
+{
+    if (thread->wait_for == WAIT_FD) {
+        /* The thread is blocked on thread->fd for read. */
+        source_prepare_add_poll_fd(source, thread->fd, G_IO_IN);
+        return;
+    }
+
+    if (thread->wait_for & WAIT_SELECT) {
+        /* thread->fd is the maximum fd of the fds in the various sets. Need to
+         * check the sets to see which fd's to wait for */
+        int fd;
+
+        for (fd = 0; fd < thread->fd; fd++) {
+            gushort events = 0;
+
+            if (FD_ISSET(fd, &thread->readfds))
+                events |= G_IO_IN;
+            if (FD_ISSET(fd, &thread->writefds))
+                events |= G_IO_OUT;
+            if (FD_ISSET(fd, &thread->exceptfds))
+                events |= G_IO_PRI | G_IO_ERR | G_IO_HUP;
+
+            if (events != 0)
+              source_prepare_add_poll_fd(source, fd, events);
+        }
+    }
+}
+
+static inline gboolean
+source_prepare_setup_poll_fd(GSource *source, gint *timeout)
+{
+    RGSource *rg_source = (RGSource *)source;
+    rb_thread_t thread;
+    gdouble now;
+
+    g_assert(rg_source->old_poll_fds == NULL);
+    rg_source->old_poll_fds = rg_source->poll_fds;
+    rg_source->poll_fds = NULL;
+
+    now = timeofday();
+    thread = rb_curr_thread;
+    do {
+        thread = thread->next;
+
+        if ((thread->wait_for == 0 && thread->status == THREAD_RUNNABLE &&
+             thread != rb_curr_thread) ||
+            (thread->wait_for & WAIT_JOIN &&
+             thread->join->status == THREAD_KILLED)) {
+            rg_source->poll_fds = g_list_concat(rg_source->poll_fds,
+                                                rg_source->old_poll_fds);
+            rg_source->old_poll_fds = NULL;
+            return TRUE;
+        }
+
+        if (thread->wait_for & WAIT_TIME && thread->delay != DELAY_INFTY) {
+            gint delay;
+
+            delay = (thread->delay - now) * 1000;
+            if (delay <= 0) {
+                rg_source->poll_fds = g_list_concat(rg_source->poll_fds,
+                                                    rg_source->old_poll_fds);
+                rg_source->old_poll_fds = NULL;
+                return TRUE;
+            }
+            if (*timeout == -1 || delay < *timeout)
+                *timeout = delay;
+        }
+
+        if (thread->wait_for == WAIT_FD || thread->wait_for & WAIT_SELECT)
+            source_prepare_add_poll(source, thread);
+    } while (thread != rb_curr_thread);
+
+    source_cleanup_poll_fds(source);
+
+    return FALSE;
+}
+
+static gboolean
+source_prepare(GSource *source, gint *timeout)
+{
+    RGSource *rg_source = (RGSource *)source;
+
+    *timeout = -1;
+    rg_source->ready = source_prepare_setup_poll_fd(source, timeout);
+
+    return rg_source->ready;
+}
+
+static gboolean
+source_check(GSource *source)
+{
+    RGSource *rg_source = (RGSource *)source;
+    GList *node;
+
+    if (rg_source->ready)
+        return TRUE;
+
+    for (node = rg_source->poll_fds; node; node = g_list_next(node)) {
+        GPollFD *poll_fd = node->data;
+
+        if (poll_fd->revents)
+            return TRUE;
+    }
+
+    return FALSE;
+}
+
+static gboolean
+source_dispatch(GSource *source, GSourceFunc callback, gpointer user_data)
+{
+    TRAP_BEG;
+    rb_thread_schedule();
+    TRAP_END;
+
+    return TRUE;
+}
+
+static void
+source_finalize(GSource *source)
+{
+    RGSource *rg_source = (RGSource *)source;
+    GList *node;
+
+    for (node = rg_source->old_poll_fds; node; node = g_list_next(node)) {
+        GPollFD *poll_fd = node->data;
+        g_slice_free(GPollFD, poll_fd);
+    }
+
+    for (node = rg_source->poll_fds; node; node = g_list_next(node)) {
+        GPollFD *poll_fd = node->data;
+        g_slice_free(GPollFD, poll_fd);
+    }
+
+    g_list_free(rg_source->old_poll_fds);
+    rg_source->old_poll_fds = NULL;
+
+    g_list_free(rg_source->poll_fds);
+    rg_source->poll_fds = NULL;
+}
+
+static GSourceFuncs source_funcs = {
+    source_prepare,
+    source_check,
+    source_dispatch,
+    source_finalize
+};
+
+static GSource *
+ruby_source_new(void)
+{
+    GSource *source;
+    RGSource *rg_source;
+
+    source = g_source_new(&source_funcs, sizeof(RGSource));
+    g_source_set_can_recurse(source, TRUE);
+
+    rg_source = (RGSource *)source;
+    rg_source->poll_fds = NULL;
+    rg_source->old_poll_fds = NULL;
+
+    return source;
+}
+#endif
+
 static VALUE
 source_remove(self, tag)
     VALUE self, tag;
@@ -54,7 +361,7 @@ invoke_source_func(data)
     callback_info_t *info = (callback_info_t *)data;
     gboolean ret;
 
-    ret = RTEST(rb_funcall(info->callback, id_call, 0));
+    ret = RVAL2CBOOL(rb_funcall(info->callback, id_call, 0));
     if (!ret)
         G_REMOVE_RELATIVE(mGLibSource, id__callbacks__, UINT2NUM(info->id));
     return ret;
@@ -80,7 +387,21 @@ static VALUE
 mc_initialize(self)
     VALUE self;
 {
-    G_INITIALIZE(self, g_main_context_new());
+    GMainContext *context;
+
+    context = g_main_context_new();
+
+    g_main_context_set_poll_func(context, rg_poll);
+#ifndef HAVE_RB_THREAD_BLOCKING_REGION
+    {
+        GSource *source;
+        source = ruby_source_new();
+        g_source_attach(source, context);
+        g_source_unref(source);
+    }
+#endif
+
+    G_INITIALIZE(self, context);
     return Qnil;
 }
 
@@ -95,7 +416,7 @@ static VALUE
 mc_iteration(self, may_block)
     VALUE self, may_block;
 {
-    return CBOOL2RVAL(g_main_context_iteration(_SELF(self), RTEST(may_block)));
+    return CBOOL2RVAL(g_main_context_iteration(_SELF(self), RVAL2CBOOL(may_block)));
 }
 
 static VALUE
@@ -253,7 +574,7 @@ static VALUE
 mc_set_poll_func(self)
     VALUE self;
 {
-    rb_ivar_set(self, id_poll_func, G_BLOCK_PROC());
+    rb_ivar_set(self, id_poll_func, rb_block_proc());
     g_main_context_set_poll_func(_SELF(self), (GPollFunc)poll_func);
 
     return self;
@@ -316,7 +637,7 @@ timeout_add(int argc, VALUE *argv, VALUE self)
 
     rb_scan_args(argc, argv, "11&", &interval, &rb_priority, &func);
 
-    priority = NIL_P(rb_priority) ? G_PRIORITY_DEFAULT : INT2NUM(rb_priority);
+    priority = NIL_P(rb_priority) ? G_PRIORITY_DEFAULT : NUM2INT(rb_priority);
     info = ALLOC(callback_info_t);
     info->callback = func;
     id = g_timeout_add_full(priority, NUM2UINT(interval),
@@ -331,19 +652,23 @@ timeout_add(int argc, VALUE *argv, VALUE self)
 
 #if GLIB_CHECK_VERSION(2,14,0)
 static VALUE
-timeout_add_seconds(VALUE self, VALUE interval)
+timeout_add_seconds(int argc, VALUE *argv, VALUE self)
 {
-    VALUE func, rb_id;
+    VALUE interval, rb_priority, func, rb_id;
+    gint priority;
     callback_info_t *info;
     guint id;
 
-    func = G_BLOCK_PROC();
+    rb_scan_args(argc, argv, "11&", &interval, &rb_priority, &func);
 
+    priority = NIL_P(rb_priority) ? G_PRIORITY_DEFAULT : NUM2INT(rb_priority);
     info = ALLOC(callback_info_t);
     info->callback = func;
-    id = g_timeout_add_seconds(NUM2UINT(interval),
-                               (GSourceFunc)invoke_source_func,
-                               (gpointer)info, g_free);
+    id = g_timeout_add_seconds_full(priority,
+                                    NUM2UINT(interval),
+                                    (GSourceFunc)invoke_source_func,
+                                    (gpointer)info,
+                                    g_free);
     info->id = id;
     rb_id = UINT2NUM(id);
     G_RELATIVE2(mGLibSource, func, id__callbacks__, rb_id);
@@ -372,15 +697,15 @@ idle_add(argc, argv, self)
 
     rb_scan_args(argc, argv, "02", &arg1, &arg2);
 
-    if (RTEST(rb_obj_is_kind_of(arg1, rb_cProc))) {
+    if (RVAL2CBOOL(rb_obj_is_kind_of(arg1, rb_cProc))) {
         priority = G_PRIORITY_DEFAULT;
         func = arg1;
-    } else if (RTEST(rb_obj_is_kind_of(arg1, rb_cInteger))) {
+    } else if (RVAL2CBOOL(rb_obj_is_kind_of(arg1, rb_cInteger))) {
         priority = NUM2INT(arg1);
-        func = G_BLOCK_PROC();
+        func = rb_block_proc();
     } else {
         priority = G_PRIORITY_DEFAULT;
-        func = G_BLOCK_PROC();
+        func = rb_block_proc();
     }
 
     info = ALLOC(callback_info_t);
@@ -427,13 +752,21 @@ static VALUE
 child_watch_add(self, pid)
     VALUE self, pid;
 {
-    VALUE func = G_BLOCK_PROC();
+    VALUE func = rb_block_proc();
     G_RELATIVE(self, func);
     return UINT2NUM(g_child_watch_add((GPid)NUM2INT(pid), 
                                       (GChildWatchFunc)child_watch_func, (gpointer)func));
 }
 #endif
 
+#ifndef HAVE_RB_THREAD_BLOCKING_REGION
+static void
+ruby_source_remove(VALUE tag)
+{
+    g_source_remove(NUM2UINT(tag));
+}
+#endif
+
 void
 Init_glib_main_context()
 {
@@ -488,7 +821,7 @@ Init_glib_main_context()
 #endif
     rb_define_module_function(timeout, "add", timeout_add, -1);
 #if GLIB_CHECK_VERSION(2,14,0)
-    rb_define_module_function(timeout, "add_seconds", timeout_add_seconds, 1);
+    rb_define_module_function(timeout, "add_seconds", timeout_add_seconds, -1);
 #endif
     rb_define_module_function(idle, "source_new", idle_source_new, 0);
     rb_define_module_function(idle, "add", idle_add, -1);
@@ -498,4 +831,21 @@ Init_glib_main_context()
     rb_define_module_function(child_watch, "source_new", child_watch_source_new, 1);
     rb_define_module_function(child_watch, "add", child_watch_add, 1);
 #endif
+
+
+    default_poll_func = g_main_context_get_poll_func(NULL);
+    g_main_context_set_poll_func(NULL, rg_poll);
+    rb_set_end_proc(restore_poll_func, Qnil);
+
+#ifndef HAVE_RB_THREAD_BLOCKING_REGION
+    {
+        GSource *source;
+        guint tag;
+
+        source = ruby_source_new();
+        tag = g_source_attach(source, NULL);
+        g_source_unref(source);
+        rb_set_end_proc(ruby_source_remove, UINT2NUM(tag));
+    }
+#endif
 }
