@@ -3,16 +3,16 @@
 
   rbgobj_type.c -
 
-  $Author: mutoh $
-  $Date: 2006/06/18 13:42:33 $
+  $Author: sakai $
+  $Date: 2007/07/16 03:35:53 $
   created at: Sun Jun  9 20:31:47 JST 2002
- 
-  Copyright (C) 2002-2006  Ruby-GNOME2 Project Team
+
+  Copyright (C) 2002-2008  Ruby-GNOME2 Project Team
   Copyright (C) 2002,2003  Masahiro Sakai
 
 **********************************************************************/
 
-#include "global.h"
+#include "rbgprivate.h"
 
 /**********************************************************************/
 /* Type Mapping */
@@ -21,8 +21,13 @@
 #include "dl.h"
 #endif
 
+static VALUE rb_cMutex;
+static VALUE lookup_class_mutex;
+
 static ID id_new;
 static ID id_superclass;
+static ID id_lock;
+static ID id_unlock;
 static VALUE gtype_to_cinfo;
 static VALUE klass_to_cinfo;
 
@@ -35,6 +40,12 @@ typedef struct {
     int flags; /* RGObjClassFlag */
 } RGObjClassInfoDynamic;
 
+typedef struct {
+    VALUE parent;
+    GType gtype;
+    gboolean create_class;
+} RGObjClassByGtypeData;
+
 static void
 cinfo_mark(RGObjClassInfo* cinfo)
 {
@@ -63,13 +74,21 @@ rbgobj_lookup_class(klass)
         return rbgobj_lookup_class(super);
     }
 
-    rb_raise(rb_eRuntimeError, "can't get gobject class information");    
+    rb_raise(rb_eRuntimeError, "can't get gobject class information");
 }
 
+static const RGObjClassInfo *rbgobj_lookup_class_by_gtype_without_lock(GType gtype,
+								       VALUE parent,
+								       gboolean create_class);
+
 static VALUE
-get_superclass(gtype)
-    GType gtype;
+get_superclass(GType gtype)
 {
+    VALUE super_class;
+
+    if (rbgobj_convert_get_superclass(gtype, &super_class))
+        return super_class;
+
     switch (gtype) {
       case G_TYPE_PARAM:
       case G_TYPE_OBJECT:
@@ -86,37 +105,49 @@ get_superclass(gtype)
       case G_TYPE_FLAGS:
         return rb_cObject;
       default:
-        if (rbgobj_fund_has_type(gtype)) {
-          return rbgobj_fund_get_superclass(gtype);
-        } else {
-            const RGObjClassInfo* cinfo_super =
-                rbgobj_lookup_class_by_gtype(g_type_parent(gtype), Qnil);
-            return cinfo_super->klass;
-        }
+      {
+          const RGObjClassInfo *cinfo_super;
+	  GType parent_type;
+
+	  parent_type = g_type_parent(gtype);
+	  cinfo_super = rbgobj_lookup_class_by_gtype_without_lock(parent_type,
+								  Qnil,
+								  TRUE);
+          return cinfo_super->klass;
+      }
     }
 }
 
-const RGObjClassInfo *
-rbgobj_lookup_class_by_gtype(gtype, parent)
-    GType gtype;
-    VALUE parent;
+static const RGObjClassInfo *
+rbgobj_lookup_class_by_gtype_without_lock(GType gtype, VALUE parent,
+					  gboolean create_class)
 {
+    GType fundamental_type;
     RGObjClassInfo* cinfo;
-    RGObjClassInfoDynamic* cinfod; 
+    RGObjClassInfoDynamic* cinfod;
     void* gclass = NULL;
-    VALUE c = rb_hash_aref(gtype_to_cinfo, INT2NUM(gtype));
+    VALUE c;
 
-    if (!NIL_P(c)){
+    if (gtype == G_TYPE_INVALID)
+        return NULL;
+
+    c = rb_hash_aref(gtype_to_cinfo, INT2NUM(gtype));
+    if (!NIL_P(c)) {
         Data_Get_Struct(c, RGObjClassInfo, cinfo);
         return cinfo;
     }
+
+    if (!create_class)
+	return NULL;
+
     c = Data_Make_Struct(rb_cData, RGObjClassInfo, cinfo_mark, NULL, cinfo);
     cinfo->gtype = gtype;
     cinfo->mark  = NULL;
     cinfo->free  = NULL;
     cinfo->flags = 0;
 
-    switch (G_TYPE_FUNDAMENTAL(gtype)){
+    fundamental_type = G_TYPE_FUNDAMENTAL(gtype);
+    switch (fundamental_type){
     case G_TYPE_POINTER:
     case G_TYPE_BOXED:
     case G_TYPE_PARAM:
@@ -132,22 +163,20 @@ rbgobj_lookup_class_by_gtype(gtype, parent)
         break;
         
     default:
-        /* we should raise exception? */
-        if (rbgobj_fund_has_type(G_TYPE_FUNDAMENTAL(gtype))) {
-          if (NIL_P(parent)) parent = get_superclass(gtype);
-          cinfo->klass = rb_funcall(rb_cClass, id_new, 1, parent);
-        } else {
+      if (NIL_P(parent)) parent = get_superclass(gtype);
+      if (NIL_P(parent)) {
           fprintf(stderr,
                   "%s: %s's fundamental type %s isn't supported\n",
                   "rbgobj_lookup_class_by_gtype",
                   g_type_name(gtype),
-                  g_type_name(G_TYPE_FUNDAMENTAL(gtype)));
+                  g_type_name(fundamental_type));
           return NULL;
-          }
-    
+      }
+      cinfo->klass = rb_funcall(rb_cClass, id_new, 1, parent);
     }
 
-    cinfod = (RGObjClassInfoDynamic*)g_hash_table_lookup(dynamic_gtype_list, g_type_name(gtype));
+    cinfod = (RGObjClassInfoDynamic *)g_hash_table_lookup(dynamic_gtype_list,
+							  g_type_name(gtype));
     if (cinfod){
         cinfo->mark = cinfod->mark;
         cinfo->free = cinfod->free;
@@ -162,39 +191,106 @@ rbgobj_lookup_class_by_gtype(gtype, parent)
     
     if (G_TYPE_IS_INSTANTIATABLE(gtype) || G_TYPE_IS_INTERFACE(gtype))
         rbgobj_define_action_methods(cinfo->klass);
-    
+
     if (G_TYPE_IS_INSTANTIATABLE(gtype)){
         GType* interfaces = NULL;
         guint n_interfaces = 0;
         int i;
-        
+
         interfaces = g_type_interfaces(gtype, &n_interfaces);
         for (i = 0; i < n_interfaces; i++){
-            rb_include_module(
-                cinfo->klass,
-                rbgobj_lookup_class_by_gtype(interfaces[i], Qnil)->klass);
+	    const RGObjClassInfo *iface_cinfo;
+	    iface_cinfo =
+		rbgobj_lookup_class_by_gtype_without_lock(interfaces[i],
+							  Qnil,
+							  TRUE);
+            rb_include_module(cinfo->klass, iface_cinfo->klass);
         }
         g_free(interfaces);
     }
-    
-    if (G_TYPE_FUNDAMENTAL(gtype) == G_TYPE_OBJECT) {
-        rbgobj_define_property_accessors(cinfo->klass);
-    } else if (G_TYPE_FUNDAMENTAL(gtype) == G_TYPE_ENUM) {
-        rbgobj_init_enum_class(cinfo->klass);
-    } else if (G_TYPE_FUNDAMENTAL(gtype) == G_TYPE_FLAGS) {
-        rbgobj_init_flags_class(cinfo->klass);
-    } else if (G_TYPE_FUNDAMENTAL(gtype) == G_TYPE_INTERFACE) {
-        rbgobj_init_interface(cinfo->klass);
-    } else {
-      rbgobj_fund_type_init_hook(G_TYPE_FUNDAMENTAL(gtype), cinfo->klass);
+
+    if (!rbgobj_convert_type_init_hook(gtype, cinfo->klass)) {
+        switch (fundamental_type) {
+          case G_TYPE_OBJECT:
+            rbgobj_init_object_class(cinfo->klass);
+            break;
+          case G_TYPE_ENUM:
+            rbgobj_init_enum_class(cinfo->klass);
+            break;
+          case G_TYPE_FLAGS:
+            rbgobj_init_flags_class(cinfo->klass);
+            break;
+          case G_TYPE_INTERFACE:
+            rbgobj_init_interface(cinfo->klass);
+            break;
+          default:
+            rbgobj_convert_type_init_hook(fundamental_type, cinfo->klass);
+            break;
+        }
     }
-    
+
     if (gclass)
         g_type_class_unref(gclass);
 
     return cinfo;
 }
 
+static VALUE
+rbgobj_lookup_class_by_gtype_body(VALUE data)
+{
+    RGObjClassByGtypeData *cdata = (RGObjClassByGtypeData *)data;
+    const RGObjClassInfo *cinfo;
+
+    cinfo = rbgobj_lookup_class_by_gtype_without_lock(cdata->gtype,
+						      cdata->parent,
+						      cdata->create_class);
+    return (VALUE)cinfo;
+}
+
+static VALUE
+rbgobj_lookup_class_by_gtype_ensure(VALUE data)
+{
+    rb_funcall(lookup_class_mutex, id_unlock, 0);
+    return Qundef;
+}
+
+const RGObjClassInfo *
+rbgobj_lookup_class_by_gtype(GType gtype, VALUE parent)
+{
+    return rbgobj_lookup_class_by_gtype_full(gtype, parent, TRUE);
+}
+
+const RGObjClassInfo *
+rbgobj_lookup_class_by_gtype_full(GType gtype, VALUE parent,
+				  gboolean create_class)
+{
+    RGObjClassByGtypeData data;
+
+    data.gtype = gtype;
+    data.parent = parent;
+    data.create_class = create_class;
+
+    if (create_class) {
+	rb_funcall(lookup_class_mutex, id_lock, 0);
+	return (RGObjClassInfo *)rb_ensure(rbgobj_lookup_class_by_gtype_body,
+					   (VALUE)&data,
+					   rbgobj_lookup_class_by_gtype_ensure,
+					   (VALUE)&data);
+    } else {
+	return rbgobj_lookup_class_by_gtype_without_lock(gtype, parent,
+							 create_class);
+    }
+}
+
+VALUE
+rbgobj_gtype_to_ruby_class(GType gtype)
+{
+    const RGObjClassInfo *cinfo;
+
+    cinfo = GTYPE2CINFO(gtype);
+    return cinfo ? cinfo->klass : Qnil;
+}
+
 VALUE
 rbgobj_define_class(gtype, name, module, mark, free, parent)
     GType gtype;
@@ -319,7 +415,7 @@ GType
 rbgobj_gtype_get(self)
     VALUE self;
 {
-    if (RTEST(rb_obj_is_kind_of(self, rbgobj_cType))) {
+    if (RVAL2CBOOL(rb_obj_is_kind_of(self, rbgobj_cType))) {
         return NUM2ULONG(rb_ivar_get(self, id_gtype));
     } else {
         return CLASS2GTYPE(self);
@@ -333,13 +429,12 @@ type_initialize(self, type)
 {
     GType gtype;
 
-    if (RTEST(rb_obj_is_kind_of(type, rb_cInteger))) {
+    if (RVAL2CBOOL(rb_obj_is_kind_of(type, rb_cInteger))) {
         gtype = NUM2UINT(type);
         // XXX
         if (!g_type_name(gtype))
             gtype = G_TYPE_INVALID;
     } else {
-        StringValue(type);
         gtype = g_type_from_name(StringValuePtr(type));
     }
 
@@ -370,7 +465,7 @@ static VALUE
 type_compare(self, other)
     VALUE self, other;
 {
-    if (!RTEST(rb_obj_is_kind_of(other, rbgobj_cType)))
+    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, rbgobj_cType)))
         return Qnil;
     else {
         GType a = rbgobj_gtype_get(self);
@@ -391,12 +486,12 @@ static VALUE
 type_eq(self, other)
     VALUE self, other;
 {
-    if (!RTEST(rb_obj_is_kind_of(other, rbgobj_cType)))
+    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, rbgobj_cType)))
         return Qnil;
     else {
         GType a = rbgobj_gtype_get(self);
         GType b = rbgobj_gtype_get(other);
-        return (a == b) ? Qtrue : Qfalse;
+        return CBOOL2RVAL(a == b);
     }
 }
 
@@ -404,12 +499,12 @@ static VALUE
 type_lt_eq(self, other)
     VALUE self, other;
 {
-    if (!RTEST(rb_obj_is_kind_of(other, rbgobj_cType)))
+    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, rbgobj_cType)))
         return Qnil;
     else {
         GType a = rbgobj_gtype_get(self);
         GType b = rbgobj_gtype_get(other);
-        return g_type_is_a(a, b) ? Qtrue : Qfalse;
+        return CBOOL2RVAL(g_type_is_a(a, b));
     }
 }
 
@@ -417,12 +512,12 @@ static VALUE
 type_gt_eq(self, other)
     VALUE self, other;
 {
-    if (!RTEST(rb_obj_is_kind_of(other, rbgobj_cType)))
+    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, rbgobj_cType)))
         return Qnil;
     else {
         GType a = rbgobj_gtype_get(self);
         GType b = rbgobj_gtype_get(other);
-        return g_type_is_a(b, a) ? Qtrue : Qfalse;
+        return CBOOL2RVAL(g_type_is_a(b, a));
     }
 }
 
@@ -430,12 +525,12 @@ static VALUE
 type_lt(self, other)
     VALUE self, other;
 {
-    if (!RTEST(rb_obj_is_kind_of(other, rbgobj_cType)))
+    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, rbgobj_cType)))
         return Qnil;
     else {
         GType a = rbgobj_gtype_get(self);
         GType b = rbgobj_gtype_get(other);
-        return (g_type_is_a(a, b) && a != b) ? Qtrue : Qfalse;
+        return CBOOL2RVAL(g_type_is_a(a, b) && a != b);
     }
 }
 
@@ -443,12 +538,12 @@ static VALUE
 type_gt(self, other)
     VALUE self, other;
 {
-    if (!RTEST(rb_obj_is_kind_of(other, rbgobj_cType)))
+    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, rbgobj_cType)))
         return Qnil;
     else {
         GType a = rbgobj_gtype_get(self);
         GType b = rbgobj_gtype_get(other);
-        return (g_type_is_a(b, a) && a != b) ? Qtrue : Qfalse;
+        return CBOOL2RVAL(g_type_is_a(b, a) && a != b);
     }
 }
 
@@ -477,77 +572,77 @@ static VALUE
 type_is_fundamental(self)
     VALUE self;
 {
-    return G_TYPE_IS_FUNDAMENTAL(rbgobj_gtype_get(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(G_TYPE_IS_FUNDAMENTAL(rbgobj_gtype_get(self)));
 }
 
 static VALUE
 type_is_derived(self)
     VALUE self;
 {
-    return G_TYPE_IS_DERIVED(rbgobj_gtype_get(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(G_TYPE_IS_DERIVED(rbgobj_gtype_get(self)));
 }
 
 static VALUE
 type_is_interface(self)
     VALUE self;
 {
-    return G_TYPE_IS_INTERFACE(rbgobj_gtype_get(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(G_TYPE_IS_INTERFACE(rbgobj_gtype_get(self)));
 }
 
 static VALUE
 type_is_classed(self)
     VALUE self;
 {
-    return G_TYPE_IS_CLASSED(rbgobj_gtype_get(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(G_TYPE_IS_CLASSED(rbgobj_gtype_get(self)));
 }
 
 static VALUE
 type_is_instantiatable(self)
     VALUE self;
 {
-    return G_TYPE_IS_INSTANTIATABLE(rbgobj_gtype_get(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(G_TYPE_IS_INSTANTIATABLE(rbgobj_gtype_get(self)));
 }
 
 static VALUE
 type_is_derivable(self)
     VALUE self;
 {
-    return G_TYPE_IS_DERIVABLE(rbgobj_gtype_get(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(G_TYPE_IS_DERIVABLE(rbgobj_gtype_get(self)));
 }
 
 static VALUE
 type_is_deep_derivable(self)
     VALUE self;
 {
-    return G_TYPE_IS_DEEP_DERIVABLE(rbgobj_gtype_get(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(G_TYPE_IS_DEEP_DERIVABLE(rbgobj_gtype_get(self)));
 }
 
 static VALUE
 type_is_abstract(self)
     VALUE self;
 {
-    return G_TYPE_IS_ABSTRACT(rbgobj_gtype_get(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(G_TYPE_IS_ABSTRACT(rbgobj_gtype_get(self)));
 }
 
 static VALUE
 type_is_value_abstract(self)
     VALUE self;
 {
-    return G_TYPE_IS_VALUE_ABSTRACT(rbgobj_gtype_get(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(G_TYPE_IS_VALUE_ABSTRACT(rbgobj_gtype_get(self)));
 }
 
 static VALUE
 type_is_value_type(self)
     VALUE self;
 {
-    return G_TYPE_IS_VALUE_TYPE(rbgobj_gtype_get(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(G_TYPE_IS_VALUE_TYPE(rbgobj_gtype_get(self)));
 }
 
 static VALUE
 type_has_value_table(self)
     VALUE self;
 {
-    return G_TYPE_HAS_VALUE_TABLE(rbgobj_gtype_get(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(G_TYPE_HAS_VALUE_TABLE(rbgobj_gtype_get(self)));
 }
 
 static VALUE
@@ -585,7 +680,7 @@ static VALUE
 type_is_a(self, is_a_type)
     VALUE self, is_a_type;
 {
-    return g_type_is_a(rbgobj_gtype_get(self), rbgobj_gtype_get(is_a_type)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(g_type_is_a(rbgobj_gtype_get(self), rbgobj_gtype_get(is_a_type)));
 }
 
 #if 0
@@ -664,6 +759,12 @@ void _def_fundamental_type(VALUE ary, GType gtype, const char* name)
 static void
 Init_type()
 {
+    rb_cMutex = rb_const_get(rb_cObject, rb_intern("Mutex"));
+    id_lock = rb_intern("lock");
+    id_unlock = rb_intern("unlock");
+    lookup_class_mutex = rb_funcall(rb_cMutex, id_new, 0);
+    rb_iv_set(mGLib, "lookup_class_mutex", lookup_class_mutex);
+
     dynamic_gtype_list = g_hash_table_new(g_str_hash, g_str_equal);
     id_gtype = rb_intern("__gobject_gtype__");
 
