@@ -1,5 +1,5 @@
 /* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */
-/* $Id: rbgnome-program.c,v 1.12 2005/09/24 18:02:43 mutoh Exp $ */
+/* $Id: rbgnome-program.c,v 1.13 2007/07/13 16:07:30 ggc Exp $ */
 /* based on libgnome/gnome-program.h */
 
 /* Gnome::Program module for Ruby/GNOME2
@@ -110,12 +110,12 @@ prepare_pdata(hash, data, popt_tablep)
         return 0;
     Check_Type(hash, T_HASH);
     ary = rb_funcall(hash, id_to_a, 0);
-    if (RARRAY(ary)->len >= SIZE_OF_PROP_TABLE)
+    if (RARRAY_LEN(ary) >= SIZE_OF_PROP_TABLE)
         rb_raise(rb_eArgError, "too many properties");
     *popt_tablep = Qnil;
-    for (i = 0; i < RARRAY(ary)->len; i++) {
-        name = RARRAY(RARRAY(ary)->ptr[i])->ptr[0];
-        value = RARRAY(RARRAY(ary)->ptr[i])->ptr[1];
+    for (i = 0; i < RARRAY_LEN(ary); i++) {
+        name = RARRAY_PTR(RARRAY_PTR(ary)[i])[0];
+        value = RARRAY_PTR(RARRAY_PTR(ary)[i])[1];
         if (SYMBOL_P(name)) {
             g_strlcpy(cname, rb_id2name(SYM2ID(name)), sizeof(cname));
         } else {
@@ -132,7 +132,7 @@ prepare_pdata(hash, data, popt_tablep)
                     break;
                   case PROP_BOOLEAN:
                     data[i].type = PROP_DATA_INT;
-                    data[i].value.i = RTEST(value) ? 1 : 0;
+                    data[i].value.i = RVAL2CBOOL(value) ? 1 : 0;
                     break;
                   case PROP_STRING:
                     data[i].type = PROP_DATA_PTR;
@@ -296,7 +296,6 @@ program_initialize(argc, argv, self)
     char **cargv;
     struct prop_data pdata[SIZE_OF_PROP_TABLE];
     VALUE popt_table;
-    EXTERN VALUE rb_progname;
     int i;
 
     rb_scan_args(argc, argv, "24", &app_id, &app_version, &module_info, &arg0, &args, &prop);
@@ -308,7 +307,7 @@ program_initialize(argc, argv, self)
 
     /* prepare arguments */
     if (NIL_P(arg0))
-        arg0 = rb_progname;
+        arg0 = rb_argv0;
     if (NIL_P(args)) {
         args = rb_argv;
     } else {
@@ -317,10 +316,10 @@ program_initialize(argc, argv, self)
 
     args = rb_obj_dup(args); /* clone to attach to 'self'. */
     rb_ary_unshift(args, arg0);
-    cargc = RARRAY(args)->len;
+    cargc = RARRAY_LEN(args);
     cargv = ALLOCA_N(char*, cargc + 1);
     for (i = 0; i < cargc; i++) {
-        cargv[i] = RVAL2CSTR(RARRAY(args)->ptr[i]);
+        cargv[i] = RVAL2CSTR(RARRAY_PTR(args)[i]);
     }
     cargv[i] = NULL;
     rb_ivar_set(self, id_args, args);
@@ -383,9 +382,9 @@ program_locate_file(argc, argv, self)
     result = gnome_program_locate_file(NIL_P(self) ? NULL : _SELF(self),
                                        RVAL2GENUM(domain, GNOME_TYPE_FILE_DOMAIN),
                                        RVAL2CSTR(file_name),
-                                       RTEST(only_if_exists),
-                                       RTEST(multi) ? &ret_locations : NULL);
-    if (RTEST(multi)) {
+                                       RVAL2CBOOL(only_if_exists),
+                                       RVAL2CBOOL(multi) ? &ret_locations : NULL);
+    if (RVAL2CBOOL(multi)) {
         obj = rb_ary_new();
         for (list = ret_locations; list != NULL; list = list->next) {
             rb_ary_push(obj, CSTR2RVAL(list->data));
@@ -454,7 +453,7 @@ program_print_common(self, arg)
     VALUE popt_table = rb_ivar_get(self, id_popt_table);
     char *argv[3];
 
-    argv[0] = RVAL2CSTR(RARRAY(args)->ptr[0]);
+    argv[0] = RVAL2CSTR(RARRAY_PTR(args)[0]);
     argv[1] = (char *)arg;
     argv[2] = NULL;
     /* ugly hack!
