@@ -3,8 +3,8 @@
 
   rbgp-gnome-print.c -
 
-  $Author: ktou $
-  $Date: 2005/10/14 04:33:08 $
+  $Author: ggc $
+  $Date: 2007/07/13 16:07:30 $
 
   Copyright (C) 2005 Ruby-GNOME2 Project Team
   Copyright (C) 2004 Kouhei Sutou <kou@cozmixng.org>
@@ -121,13 +121,13 @@ static gdouble *
 rb_array_to_gdouble_array(VALUE array)
 {
     int i;
-    gint size = RARRAY(array)->len;
+    gint size = RARRAY_LEN(array);
     gdouble *g_array;
 
     g_array = ALLOC_N(gdouble, size);
 
     for (i = 0; i < size; i++) {
-        g_array[i] = NUM2DBL(RARRAY(array)->ptr[i]);
+        g_array[i] = NUM2DBL(RARRAY_PTR(array)[i]);
     }
 
     return g_array;
@@ -225,7 +225,7 @@ _gp_bpath(VALUE self, VALUE bpath, VALUE append)
 {
     rbgp_check_return_code(gnome_print_bpath(_SELF(self),
                                              get_art_bpath(bpath),
-                                             RTEST(append)));
+                                             RVAL2CBOOL(append)));
     return self;
 }
 
@@ -246,7 +246,7 @@ _gp_vpath(VALUE self, VALUE vpath, VALUE append)
 {
     rbgp_check_return_code(gnome_print_vpath(_SELF(self),
                                              get_art_vpath(vpath),
-                                             RTEST(append)));
+                                             RVAL2CBOOL(append)));
     return self;
 }
 
@@ -290,7 +290,7 @@ gp_arcto(VALUE self, VALUE x, VALUE y, VALUE radius,
                                              NUM2DBL(radius),
                                              NUM2DBL(angle1),
                                              NUM2DBL(angle2),
-                                             RTEST(clockwise) ? 1 : 0));
+                                             RVAL2CBOOL(clockwise) ? 1 : 0));
     return self;
 }
 
@@ -350,7 +350,7 @@ gp_setdash(VALUE self, VALUE values, VALUE offset)
     gint ret;
     gdouble *g_values = rb_array_to_gdouble_array(values);
     ret = gnome_print_setdash(_SELF(self),
-                              RARRAY(values)->len,
+                              RARRAY_LEN(values),
                               g_values,
                               NUM2INT(offset));
     free(g_values);
