@@ -21,6 +21,9 @@ PKGConfig.have_package('gthread-2.0')
 PKGConfig.have_package(PKG_CONFIG_ID) or exit 1
 setup_win32(PACKAGE_NAME)
 
+have_header("st.h")
+have_header("ruby/st.h")
+
 STDOUT.print("checking for target... ")
 STDOUT.flush
 target = PKGConfig.variable(PKG_CONFIG_ID, "target")
@@ -42,35 +45,28 @@ tmpincl.each do |i|
 end
 raise "can't find gdkkeysyms.h" if gdkincl.nil?
 
-have_func('gtk_plug_get_type')
-have_func('gtk_socket_get_type')
-have_func('pango_render_part_get_type')
-
-if target=="x11"
-  have_func("XReadBitmapFileData")
-  have_header('X11/Xlib.h')
-  have_func("XGetErrorText")
+gtk_header = "gtk/gtk.h"
+have_func('gtk_plug_get_type', gtk_header)
+have_func('gtk_socket_get_type', gtk_header)
+have_func('pango_render_part_get_type', gtk_header)
+have_header('gtk/gtkfilesystem.h') do |src|
+  "#define GTK_FILE_SYSTEM_ENABLE_UNSUPPORTED\n#{src}"
 end
 
-PKGConfig.have_package('gtk+-unix-print-2.0')
-
-have_func('gtk_print_unix_dialog_get_type')
-have_func('gtk_print_job_get_type')
-have_func('gtk_printer_get_type')
-have_func('gtk_page_setup_unix_get_type')
-
-PKGConfig.have_package('cairo')
-if have_header('rb_cairo.h')
-  if /mingw|cygwin|mswin32/ =~ RUBY_PLATFORM
-    unless ENV["CAIRO_PATH"]
-      puts "Error! Set CAIRO_PATH."
-      exit 1
-    end
-    add_depend_package("cairo", "packages/cairo/ext", ENV["CAIRO_PATH"])
-    $defs << "-DRUBY_CAIRO_PLATFORM_WIN32"
-  end
+xlib_header = "X11/Xlib.h"
+if target == "x11" and have_header('X11/Xlib.h')
+  have_func("XReadBitmapFileData", xlib_header)
+  have_func("XGetErrorText", xlib_header)
 end
 
+if PKGConfig.have_package('gtk+-unix-print-2.0')
+  $defs.push("-DHAVE_GTK_UNIX_PRINT")
+end
+
+have_func("rb_errinfo")
+
+check_cairo
+
 add_depend_package("glib2", "glib/src", TOPDIR)
 add_depend_package("pango", "pango/src", TOPDIR)
 
@@ -81,15 +77,15 @@ add_depend_package("pango", "pango/src", TOPDIR)
 add_distcleanfile("rbgdkkeysyms.h")
 add_distcleanfile("rbgtkinits.c")
 
-create_makefile_at_srcdir(PACKAGE_NAME, SRCDIR, "-DRUBY_GTK2_COMPILATION") {
-  File.delete("rbgtkinits.c") if FileTest.exist?("rbgtkinits.c")
-  File.delete("rbgtkinits.c") if FileTest.exist?("rbgtkinits.c")
-  system("#{$ruby} #{SRCDIR}/makeinits.rb #{SRCDIR}/*.c > rbgtkinits.c") or raise "failed to make GTK inits"
-  system("#{$ruby} #{SRCDIR}/makekeysyms.rb #{gdkincl}/gdkkeysyms.h > rbgdkkeysyms.h") or raise "failed to make GDK Keysyms"
-}
-
 $defs.delete("-DRUBY_GTK2_COMPILATION")
 
 add_depend_package("gtk2", "gtk/src", TOPDIR)
 
+create_makefile_at_srcdir(PACKAGE_NAME, SRCDIR, "-DRUBY_GTK2_COMPILATION") {
+  File.delete("rbgtkinits.c") if FileTest.exist?("rbgtkinits.c")
+  SRCDIR_QUOTED = SRCDIR.gsub(' ', '\ ')
+  system("#{$ruby} #{SRCDIR_QUOTED}/makeinits.rb #{SRCDIR_QUOTED}/*.c > rbgtkinits.c") or raise "failed to make GTK inits"
+  system("#{$ruby} #{SRCDIR_QUOTED}/makekeysyms.rb #{gdkincl}/gdkkeysyms.h > rbgdkkeysyms.h") or raise "failed to make GDK Keysyms"
+}
+
 create_top_makefile(["src"])
