@@ -3,8 +3,8 @@
 
   rbgtkactiongroup.c -
 
-  $Author: mutoh $
-  $Date: 2006/06/17 06:59:32 $
+  $Author: ggc $
+  $Date: 2007/07/13 16:07:31 $
 
   Copyright (C) 2004-2006 Masao Mutoh
 ************************************************/
@@ -64,6 +64,8 @@ actiongroup_add_action(argc, argv, self)
     gtk_action_group_add_action_with_accel(_SELF(self),
                                            GTK_ACTION(RVAL2GOBJ(action)),
                                            NIL_P(accelerator) ? NULL : RVAL2CSTR(accelerator));
+    G_CHILD_ADD(self, action);
+
     return self;
 }
 
@@ -72,6 +74,7 @@ actiongroup_remove_action(self, action)
     VALUE self, action;
 {
     gtk_action_group_remove_action(_SELF(self), GTK_ACTION(RVAL2GOBJ(action)));
+    G_CHILD_REMOVE(self, action);
     return self;
 }
 
@@ -92,7 +95,7 @@ actiongroup_add_actions(self, entries)
 {
     int i;
     VALUE action_procs;
-    guint n_entries = (guint)RARRAY(entries)->len;
+    guint n_entries = (guint)RARRAY_LEN(entries);
     GtkActionEntry* gentries = g_new0(GtkActionEntry, n_entries);
 
     if (rb_ivar_defined(self, id_action_procs) == Qtrue){
@@ -102,39 +105,43 @@ actiongroup_add_actions(self, entries)
     }
 
     for (i = 0; i < n_entries; i++){
-        struct RArray* entry = RARRAY(RARRAY(entries)->ptr[i]);
-        int size = entry->len;
+        VALUE entry;
+        int size;
 
+	entry = RARRAY_PTR(entries)[i];
+	size = RARRAY_LEN(entry);
         if (size < 1)
             rb_raise(rb_eArgError, "wrong array parameter");
 
-        gentries[i].name = NIL_P(entry->ptr[0]) ? NULL: RVAL2CSTR(entry->ptr[0]);
+        gentries[i].name = RVAL2CSTR_ACCEPT_NIL(RARRAY_PTR(entry)[0]);
         gentries[i].callback = G_CALLBACK(activate_action);
 
         if (size < 2) continue;
 
-        if (NIL_P(entry->ptr[1])){
+        if (NIL_P(RARRAY_PTR(entry)[1])){
             gentries[i].stock_id = NULL;
-        } else if (SYMBOL_P(entry->ptr[1])){
-            gentries[i].stock_id = rb_id2name(SYM2ID(entry->ptr[1]));
-        } else if (TYPE(entry->ptr[1]) == T_STRING){
-            gentries[i].stock_id = RVAL2CSTR(entry->ptr[1]);
+        } else if (SYMBOL_P(RARRAY_PTR(entry)[1])){
+            gentries[i].stock_id = rb_id2name(SYM2ID(RARRAY_PTR(entry)[1]));
+        } else if (TYPE(RARRAY_PTR(entry)[1]) == T_STRING){
+            gentries[i].stock_id = RVAL2CSTR(RARRAY_PTR(entry)[1]);
         } else{
-            rb_raise(rb_eArgError, "invalid argument %s (expect Symbol or String)",
-                     rb_class2name(CLASS_OF(entry->ptr[1])));
+            rb_raise(rb_eArgError,
+		     "invalid argument %s (expect Symbol or String)",
+                     rb_class2name(CLASS_OF(RARRAY_PTR(entry)[1])));
         }
 
         if (size < 3) continue;
-        gentries[i].label = NIL_P(entry->ptr[2]) ? (const gchar*)NULL : RVAL2CSTR(entry->ptr[2]);
+        gentries[i].label = RVAL2CSTR_ACCEPT_NIL(RARRAY_PTR(entry)[2]);
         if (size < 4) continue;
-        gentries[i].accelerator = NIL_P(entry->ptr[3]) ? (const gchar*)NULL : RVAL2CSTR(entry->ptr[3]);
+        gentries[i].accelerator = RVAL2CSTR_ACCEPT_NIL(RARRAY_PTR(entry)[3]);
         if (size < 4) continue;
-        gentries[i].tooltip = NIL_P(entry->ptr[4]) ? (const gchar*)NULL : RVAL2CSTR(entry->ptr[4]);
+        gentries[i].tooltip = RVAL2CSTR_ACCEPT_NIL(RARRAY_PTR(entry)[4]);
         if (size < 5) continue;
-        rb_hash_aset(action_procs, entry->ptr[0], entry->ptr[5]);
-    } 
+        rb_hash_aset(action_procs, RARRAY_PTR(entry)[0], RARRAY_PTR(entry)[5]);
+    }
     rb_ivar_set(self, id_action_procs, action_procs);
-    gtk_action_group_add_actions(_SELF(self), gentries, n_entries, (gpointer)self);
+    gtk_action_group_add_actions(_SELF(self), gentries, n_entries,
+				 (gpointer)self);
     g_free(gentries);
 
     return self;
@@ -157,7 +164,7 @@ actiongroup_add_toggle_actions(self, entries)
 {
     int i;
     VALUE toggle_action_procs;
-    guint n_entries = (guint)RARRAY(entries)->len;
+    guint n_entries = (guint)RARRAY_LEN(entries);
     GtkToggleActionEntry* gentries = g_new0(GtkToggleActionEntry, n_entries);
 
     if (rb_ivar_defined(self, id_toggle_action_procs) == Qtrue){
@@ -167,41 +174,45 @@ actiongroup_add_toggle_actions(self, entries)
     }
 
     for (i = 0; i < n_entries; i++){
-        struct RArray* entry = RARRAY(RARRAY(entries)->ptr[i]);
-        int size = entry->len;
+        VALUE entry;
+	int size;
 
+	entry = RARRAY_PTR(entries)[i];
+	size = RARRAY_LEN(entry);
         if (size < 1)
             rb_raise(rb_eArgError, "wrong array parameter");
 
-        gentries[i].name = NIL_P(entry->ptr[0]) ? NULL: RVAL2CSTR(entry->ptr[0]);
+        gentries[i].name = RVAL2CSTR_ACCEPT_NIL(RARRAY_PTR(entry)[0]);
         gentries[i].callback = G_CALLBACK(activate_toggle_action);
 
         if (size < 2) continue;
 
-        if (NIL_P(entry->ptr[1])){
+        if (NIL_P(RARRAY_PTR(entry)[1])){
             gentries[i].stock_id = NULL;
-        } else if (SYMBOL_P(entry->ptr[1])){
-            gentries[i].stock_id = rb_id2name(SYM2ID(entry->ptr[1]));
-        } else if (TYPE(entry->ptr[1]) == T_STRING){
-            gentries[i].stock_id = RVAL2CSTR(entry->ptr[1]);
+        } else if (SYMBOL_P(RARRAY_PTR(entry)[1])){
+            gentries[i].stock_id = rb_id2name(SYM2ID(RARRAY_PTR(entry)[1]));
+        } else if (TYPE(RARRAY_PTR(entry)[1]) == T_STRING){
+            gentries[i].stock_id = RVAL2CSTR(RARRAY_PTR(entry)[1]);
         } else{
             rb_raise(rb_eArgError, "invalid argument %s (expect Symbol or String)",
-                     rb_class2name(CLASS_OF(entry->ptr[1])));
+                     rb_class2name(CLASS_OF(RARRAY_PTR(entry)[1])));
         }
 
         if (size < 3) continue;
-        gentries[i].label = NIL_P(entry->ptr[2]) ? (const gchar*)NULL : RVAL2CSTR(entry->ptr[2]);
+        gentries[i].label = RVAL2CSTR_ACCEPT_NIL(RARRAY_PTR(entry)[2]);
         if (size < 4) continue;
-        gentries[i].accelerator = NIL_P(entry->ptr[3]) ? (const gchar*)NULL : RVAL2CSTR(entry->ptr[3]);
+        gentries[i].accelerator = RVAL2CSTR_ACCEPT_NIL(RARRAY_PTR(entry)[3]);
         if (size < 4) continue;
-        gentries[i].tooltip = NIL_P(entry->ptr[4]) ? (const gchar*)NULL : RVAL2CSTR(entry->ptr[4]);
+        gentries[i].tooltip = RVAL2CSTR_ACCEPT_NIL(RARRAY_PTR(entry)[4]);
         if (size < 5) continue;
-        rb_hash_aset(toggle_action_procs, entry->ptr[0], entry->ptr[5]);
+        rb_hash_aset(toggle_action_procs,
+		     RARRAY_PTR(entry)[0], RARRAY_PTR(entry)[5]);
         if (size < 6) continue;
-        gentries[i].is_active = RTEST(entry->ptr[6]);
-    } 
+        gentries[i].is_active = RVAL2CBOOL(RARRAY_PTR(entry)[6]);
+    }
     rb_ivar_set(self, id_toggle_action_procs, toggle_action_procs);
-    gtk_action_group_add_toggle_actions(_SELF(self), gentries, n_entries, (gpointer)self);
+    gtk_action_group_add_toggle_actions(_SELF(self), gentries, n_entries,
+					(gpointer)self);
     g_free(gentries);
 
     return self;
@@ -232,45 +243,50 @@ actiongroup_add_radio_actions(argc, argv, self)
 
     if (NIL_P(value)) value = -1;
     if (NIL_P(proc) && rb_block_given_p()){
-        proc = G_BLOCK_PROC();
+        proc = rb_block_proc();
         G_RELATIVE(self, proc);
     }
-    n_entries = (guint)RARRAY(entries)->len;
+    n_entries = (guint)RARRAY_LEN(entries);
     gentries = g_new0(GtkRadioActionEntry, n_entries);
 
     for (i = 0; i < n_entries; i++){
-        struct RArray* entry = RARRAY(RARRAY(entries)->ptr[i]);
-        int size = entry->len;
+        VALUE entry;
+	int size;
 
+	entry = RARRAY_PTR(entries)[i];
+	size = RARRAY_LEN(entry);
         if (size < 1)
             rb_raise(rb_eArgError, "wrong array parameter");
 
-        gentries[i].name = NIL_P(entry->ptr[0]) ? NULL: RVAL2CSTR(entry->ptr[0]);
+        gentries[i].name = RVAL2CSTR_ACCEPT_NIL(RARRAY_PTR(entry)[0]);
 
         if (size < 2) continue;
 
-        if (NIL_P(entry->ptr[1])){
+        if (NIL_P(RARRAY_PTR(entry)[1])){
             gentries[i].stock_id = NULL;
-        } else if (SYMBOL_P(entry->ptr[1])){
-            gentries[i].stock_id = rb_id2name(SYM2ID(entry->ptr[1]));
-        } else if (TYPE(entry->ptr[1]) == T_STRING){
-            gentries[i].stock_id = RVAL2CSTR(entry->ptr[1]);
+        } else if (SYMBOL_P(RARRAY_PTR(entry)[1])){
+            gentries[i].stock_id = rb_id2name(SYM2ID(RARRAY_PTR(entry)[1]));
+        } else if (TYPE(RARRAY_PTR(entry)[1]) == T_STRING){
+            gentries[i].stock_id = RVAL2CSTR(RARRAY_PTR(entry)[1]);
         } else{
-            rb_raise(rb_eArgError, "invalid argument %s (expect Symbol or String)",
-                     rb_class2name(CLASS_OF(entry->ptr[1])));
+            rb_raise(rb_eArgError,
+		     "invalid argument %s (expect Symbol or String)",
+                     rb_class2name(CLASS_OF(RARRAY_PTR(entry)[1])));
         }
 
         if (size < 3) continue;
-        gentries[i].label = NIL_P(entry->ptr[2]) ? (const gchar*)NULL : RVAL2CSTR(entry->ptr[2]);
+        gentries[i].label = RVAL2CSTR_ACCEPT_NIL(RARRAY_PTR(entry)[2]);
         if (size < 4) continue;
-        gentries[i].accelerator = NIL_P(entry->ptr[3]) ? (const gchar*)NULL : RVAL2CSTR(entry->ptr[3]);
+        gentries[i].accelerator = RVAL2CSTR_ACCEPT_NIL(RARRAY_PTR(entry)[3]);
         if (size < 4) continue;
-        gentries[i].tooltip = NIL_P(entry->ptr[4]) ? (const gchar*)NULL : RVAL2CSTR(entry->ptr[4]);
+        gentries[i].tooltip = RVAL2CSTR_ACCEPT_NIL(RARRAY_PTR(entry)[4]);
         if (size < 5) continue;
-        gentries[i].value = NUM2INT(entry->ptr[5]);
-    } 
-    gtk_action_group_add_radio_actions(_SELF(self), gentries, n_entries, NUM2INT(value), 
-                                       G_CALLBACK(activate_radio_action), (gpointer)proc);
+        gentries[i].value = NUM2INT(RARRAY_PTR(entry)[5]);
+    }
+    gtk_action_group_add_radio_actions(_SELF(self), gentries,
+				       n_entries, NUM2INT(value),
+                                       G_CALLBACK(activate_radio_action),
+				       (gpointer)proc);
     g_free(gentries);
 
     return self;
@@ -289,7 +305,7 @@ static VALUE
 actiongroup_set_translate_func(self)
     VALUE self;
 {
-    VALUE func = G_BLOCK_PROC();
+    VALUE func = rb_block_proc();
     G_RELATIVE(self, func);
     gtk_action_group_set_translate_func(_SELF(self), 
                                         (GtkTranslateFunc)translate_func, 
@@ -314,13 +330,31 @@ actiongroup_translate_string(self, str)
     return CSTR2RVAL(gtk_action_group_translate_string(_SELF(self), RVAL2CSTR(str)));
 }
 #endif
+
+static void
+action_group_mark(void *p)
+{
+    GtkActionGroup *group;
+    GList *actions, *node;
+
+    group = GTK_ACTION_GROUP(p);
+    actions = gtk_action_group_list_actions(group);
+    for (node = actions; node; node = g_list_next(node)) {
+	GtkWidget *action = node->data;
+	rbgobj_gc_mark_instance(action);
+    }
+    g_list_free(actions);
+}
 #endif
 
 void 
 Init_gtk_actiongroup()
 {
 #if GTK_CHECK_VERSION(2,4,0)
-    VALUE gActionGroup = G_DEF_CLASS(GTK_TYPE_ACTION_GROUP, "ActionGroup", mGtk);
+    VALUE gActionGroup;
+
+    gActionGroup = G_DEF_CLASS_WITH_GC_FUNC(GTK_TYPE_ACTION_GROUP, "ActionGroup",
+					    mGtk, action_group_mark, NULL);
 
     id_action_procs = rb_intern("@action_procs");
     id_toggle_action_procs = rb_intern("@toggle_action_procs");
