@@ -3,8 +3,8 @@
 
   rbgtkcolorsel.c -
 
-  $Author: mutoh $
-  $Date: 2005/03/22 15:42:55 $
+  $Author: ggc $
+  $Date: 2007/07/13 14:27:09 $
 
   Copyright (C) 2002,2003 Ruby-GNOME2 Project Team
   Copyright (C) 1998-2000 Yukihiro Matsumoto,
@@ -15,8 +15,6 @@
 #include "global.h"
 
 #define _SELF(s) (GTK_COLOR_SELECTION(RVAL2GOBJ(self)))
-#define RVAL2COLOR(c) ((GdkColor*)RVAL2BOXED(c, GDK_TYPE_COLOR))
-#define COLOR2RVAL(c) (BOXED2RVAL(c, GDK_TYPE_COLOR))
 
 static VALUE gColorSel;
 
@@ -47,7 +45,7 @@ static VALUE
 colorsel_set_previous_color(self, color)
     VALUE self, color;
 {
-    gtk_color_selection_set_previous_color(_SELF(self), RVAL2COLOR(color));
+    gtk_color_selection_set_previous_color(_SELF(self), RVAL2GDKCOLOR(color));
     return self;
 }
 
@@ -57,14 +55,14 @@ colorsel_get_previous_color(self)
 {
     GdkColor color;
     gtk_color_selection_get_previous_color(_SELF(self), &color);
-    return COLOR2RVAL(&color);
+    return GDKCOLOR2RVAL(&color);
 }
 
 static VALUE
 colorsel_is_adjusting(self)
     VALUE self;
 {
-    return gtk_color_selection_is_adjusting(_SELF(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_color_selection_is_adjusting(_SELF(self)));
 }
 
 static VALUE
@@ -80,7 +78,7 @@ colorsel_s_palette_from_string(self, str)
     if (ret) {
         ary = rb_ary_new();
         for (i = 0; i < n_colors; i++) {
-            rb_ary_push(ary, COLOR2RVAL(&gcolors[i]));
+            rb_ary_push(ary, GDKCOLOR2RVAL(&gcolors[i]));
         }
     }
                                                                                 
@@ -104,11 +102,11 @@ colorsel_s_palette_to_string(argc, argv, self)
         rb_scan_args(argc, argv, "10", &colors);
     }
 
-    len = RARRAY(colors)->len;
+    len = RARRAY_LEN(colors);
     gcolors = ALLOCA_N(GdkColor, len);
 
     for (i = 0; i < len; i++) {
-        gcolor = (GdkColor*)RVAL2BOXED(RARRAY(colors)->ptr[i], GDK_TYPE_COLOR);
+        gcolor = RVAL2GDKCOLOR(RARRAY_PTR(colors)[i]);
         gcolors[i] = *gcolor;
     }
 
@@ -126,7 +124,7 @@ screen_func(screen, colors, n_colors)
     VALUE func = rb_cvar_get(gColorSel, rb_intern("__palette_proc__"));
     VALUE ary = rb_ary_new();
     for (i = 0; i < n_colors; i++){
-        ary = rb_ary_push(ary, BOXED2RVAL((GdkColor*)&colors[i], GDK_TYPE_COLOR));
+        ary = rb_ary_push(ary, GDKCOLOR2RVAL((GdkColor *)&colors[i]));
     }
     if (! NIL_P(func))
         rb_funcall(func, id_call, 2, GOBJ2RVAL(screen), ary);
@@ -136,9 +134,9 @@ static VALUE
 colorsel_s_set_change_palette_hook(self)
     VALUE self;
 {
-    VALUE func = G_BLOCK_PROC();
+    VALUE func = rb_block_proc();
 
-    rb_cvar_set(gColorSel, rb_intern("__palette_proc__"), func, 0);
+    rb_cv_set(gColorSel, "__palette_proc__", func);
     gtk_color_selection_set_change_palette_with_screen_hook(
         (GtkColorSelectionChangePaletteWithScreenFunc)screen_func);
     return self;
