@@ -3,8 +3,8 @@
 
   rbgtktextiter.c -
 
-  $Author: ktou $
-  $Date: 2006/12/26 01:49:14 $
+  $Author: ggc $
+  $Date: 2007/07/13 16:07:32 $
 
   Copyright (C) 2002-2005 Ruby-GNOME2 Project Team
   Copyright (C) 2002,2003 Masahiro Sakai
@@ -103,7 +103,7 @@ static VALUE
 get_toggled_tags(self, toggled_on)
     VALUE self, toggled_on;
 {
-    return GSLIST2ARYF(gtk_text_iter_get_toggled_tags(_SELF(self), RTEST(toggled_on)));
+    return GSLIST2ARYF(gtk_text_iter_get_toggled_tags(_SELF(self), RVAL2CBOOL(toggled_on)));
 }
 
 static VALUE
@@ -153,14 +153,14 @@ static VALUE
 editable(self, default_setting)
     VALUE self, default_setting;
 {
-    return CBOOL2RVAL(gtk_text_iter_editable(_SELF(self), RTEST(default_setting)));
+    return CBOOL2RVAL(gtk_text_iter_editable(_SELF(self), RVAL2CBOOL(default_setting)));
 }
 
 static VALUE
 can_insert(self, default_setting)
     VALUE self, default_setting;
 {
-    return CBOOL2RVAL(gtk_text_iter_can_insert(_SELF(self), RTEST(default_setting)));
+    return CBOOL2RVAL(gtk_text_iter_can_insert(_SELF(self), RVAL2CBOOL(default_setting)));
 }
 
     
@@ -213,7 +213,7 @@ static VALUE \
 __name__(self) \
     VALUE self; \
 { \
-    return gtk_text_iter_##__name__(_SELF(self)) ? Qtrue : Qfalse; \
+    return CBOOL2RVAL(gtk_text_iter_##__name__(_SELF(self))); \
 }
 
 #define def_move_gint(__name__) \
@@ -221,7 +221,7 @@ static VALUE \
 __name__(self, i) \
     VALUE self, i; \
 { \
-    return gtk_text_iter_##__name__(_SELF(self), NUM2INT(i)) ? Qtrue : Qfalse; \
+    return CBOOL2RVAL(gtk_text_iter_##__name__(_SELF(self), NUM2INT(i))); \
 }
 
 def_move(forward_char)
@@ -297,9 +297,8 @@ forward_to_tag_toggle(argc, argv, self)
     VALUE tag;
 
     rb_scan_args(argc, argv, "01", &tag);
-    return gtk_text_iter_forward_to_tag_toggle(_SELF(self),
-                                               NIL_P(tag) ? NULL : RVAL2TAG(tag))
-        ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_text_iter_forward_to_tag_toggle(_SELF(self),
+                                                          NIL_P(tag) ? NULL : RVAL2TAG(tag)));
 }
 
 static VALUE
@@ -311,9 +310,8 @@ backward_to_tag_toggle(argc, argv, self)
     VALUE tag;
 
     rb_scan_args(argc, argv, "01", &tag);
-    return gtk_text_iter_backward_to_tag_toggle(_SELF(self),
-                                                NIL_P(tag) ? NULL : RVAL2TAG(tag))
-        ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_text_iter_backward_to_tag_toggle(_SELF(self),
+                                                           NIL_P(tag) ? NULL : RVAL2TAG(tag)));
 }
 
 static gboolean
@@ -321,7 +319,7 @@ char_predicate_func(ch, func)
     guint32 ch;
     gpointer func;
 {
-    return RTEST(rb_funcall((VALUE)func, id_call, 1, UINT2NUM(ch)));
+    return RVAL2CBOOL(rb_funcall((VALUE)func, id_call, 1, UINT2NUM(ch)));
 }
 
 static VALUE
@@ -331,14 +329,13 @@ forward_find_char(argc, argv, self)
     VALUE self;
 {
     VALUE limit;
-    volatile VALUE func = G_BLOCK_PROC();
+    volatile VALUE func = rb_block_proc();
     
     rb_scan_args(argc, argv, "01", &limit);
-    return gtk_text_iter_forward_find_char(_SELF(self),
-                                           (GtkTextCharPredicate)char_predicate_func, 
-                                           (gpointer)func,
-                                           NIL_P(limit) ? NULL : _SELF(limit))
-        ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_text_iter_forward_find_char(_SELF(self),
+                                                      (GtkTextCharPredicate)char_predicate_func, 
+                                                      (gpointer)func,
+                                                      NIL_P(limit) ? NULL : _SELF(limit)));
 }
 
 static VALUE
@@ -348,14 +345,13 @@ backward_find_char(argc, argv, self)
     VALUE self;
 {
     VALUE limit;
-    volatile VALUE func = G_BLOCK_PROC();
+    volatile VALUE func = rb_block_proc();
 
     rb_scan_args(argc, argv, "01", &limit);
-    return gtk_text_iter_backward_find_char(_SELF(self),
-                                            (GtkTextCharPredicate)char_predicate_func,
-                                            (gpointer)func,
-                                            NIL_P(limit) ? NULL : _SELF(limit))
-        ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_text_iter_backward_find_char(_SELF(self),
+                                                       (GtkTextCharPredicate)char_predicate_func,
+                                                       (gpointer)func,
+                                                       NIL_P(limit) ? NULL : _SELF(limit)));
 }
 
 static VALUE
@@ -413,7 +409,7 @@ static VALUE
 equal(self, other)
     VALUE self, other;
 {
-    return gtk_text_iter_equal(_SELF(self), _SELF(other)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_text_iter_equal(_SELF(self), _SELF(other)));
 }
 
 static VALUE
