@@ -3,8 +3,8 @@
 
   rbgtkwidget.c -
 
-  $Author: ktou $
-  $Date: 2006/07/09 13:20:06 $
+  $Author: ggc $
+  $Date: 2007/07/13 16:07:32 $
 
   Copyright (C) 2002-2006 Ruby-GNOME2 Project Team
   Copyright (C) 1998-2000 Yukihiro Matsumoto,
@@ -196,10 +196,10 @@ static VALUE
 widget_remove_accelerator(self, accel, key, mod)
     VALUE self, accel, key, mod;
 {
-    return gtk_widget_remove_accelerator(_SELF(self),
-                                         GTK_ACCEL_GROUP(RVAL2GOBJ(accel)),
-                                         NUM2INT(key),
-                                         RVAL2GFLAGS(mod, GDK_TYPE_MODIFIER_TYPE)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_widget_remove_accelerator(_SELF(self),
+                                                    GTK_ACCEL_GROUP(RVAL2GOBJ(accel)),
+                                                    NUM2INT(key),
+                                                    RVAL2GFLAGS(mod, GDK_TYPE_MODIFIER_TYPE)));
 }
 
 static VALUE
@@ -231,14 +231,14 @@ static VALUE
 widget_event(self, event)
     VALUE self, event;
 {
-    return gtk_widget_event(_SELF(self), RVAL2GEV(event)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_widget_event(_SELF(self), RVAL2GEV(event)));
 }
 
 static VALUE
 widget_activate(self)
     VALUE self;
 {
-    return gtk_widget_activate(_SELF(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_widget_activate(_SELF(self)));
 }
 
 static VALUE
@@ -264,7 +264,7 @@ static VALUE
 widget_is_focus(self)
     VALUE self;
 {
-    return gtk_widget_is_focus(_SELF(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_widget_is_focus(_SELF(self)));
 }
 
 static VALUE
@@ -287,7 +287,7 @@ static VALUE
 widget_set_state(self, state)
     VALUE self, state;
 {
-    gtk_widget_set_state(_SELF(self), RVAL2GENUM(state, GTK_TYPE_STATE_TYPE));
+    gtk_widget_set_state(_SELF(self), RVAL2GTKSTATETYPE(state));
     return self;
 }
 
@@ -365,7 +365,7 @@ static VALUE
 widget_is_ancestor(self, ancestor)
     VALUE self, ancestor;
 {
-    return (gtk_widget_is_ancestor(_SELF(self), _SELF(ancestor)) ? Qtrue : Qfalse);
+    return CBOOL2RVAL(gtk_widget_is_ancestor(_SELF(self), _SELF(ancestor)));
 }
 
 static VALUE
@@ -388,7 +388,7 @@ static VALUE
 widget_hide_on_delete(self)
 	VALUE self;
 {
-    return gtk_widget_hide_on_delete(_SELF(self)) ? Qtrue: Qfalse;
+    return CBOOL2RVAL(gtk_widget_hide_on_delete(_SELF(self)));
 }
 
 static VALUE
@@ -569,8 +569,8 @@ static VALUE
 widget_modify_fg(self, state, color)
     VALUE self, state, color;
 {
-    gtk_widget_modify_fg(_SELF(self), RVAL2GENUM(state, GTK_TYPE_STATE_TYPE), 
-                         (GdkColor*)RVAL2BOXED(color, GDK_TYPE_COLOR));
+    gtk_widget_modify_fg(_SELF(self), RVAL2GTKSTATETYPE(state),
+                         RVAL2GDKCOLOR(color));
     return self;
 }
 
@@ -578,8 +578,8 @@ static VALUE
 widget_modify_bg(self, state, color)
     VALUE self, state, color;
 {
-    gtk_widget_modify_bg(_SELF(self), RVAL2GENUM(state, GTK_TYPE_STATE_TYPE),
-                         (GdkColor*)RVAL2BOXED(color, GDK_TYPE_COLOR));
+    gtk_widget_modify_bg(_SELF(self), RVAL2GTKSTATETYPE(state),
+                         RVAL2GDKCOLOR(color));
     return self;
 }
 
@@ -587,8 +587,8 @@ static VALUE
 widget_modify_text(self, state, color)
     VALUE self, state, color;
 {
-    gtk_widget_modify_text(_SELF(self), RVAL2GENUM(state, GTK_TYPE_STATE_TYPE),
-                         (GdkColor*)RVAL2BOXED(color, GDK_TYPE_COLOR));
+    gtk_widget_modify_text(_SELF(self), RVAL2GTKSTATETYPE(state),
+                         RVAL2GDKCOLOR(color));
     return self;
 }
 
@@ -596,8 +596,8 @@ static VALUE
 widget_modify_base(self, state, color)
     VALUE self, state, color;
 {
-    gtk_widget_modify_base(_SELF(self), RVAL2GENUM(state, GTK_TYPE_STATE_TYPE),
-                         (GdkColor*)RVAL2BOXED(color, GDK_TYPE_COLOR));
+    gtk_widget_modify_base(_SELF(self), RVAL2GTKSTATETYPE(state),
+                         RVAL2GDKCOLOR(color));
     return self;
 }
 
@@ -610,6 +610,18 @@ widget_modify_font(self, font_desc)
     return self;
 }
 
+#if GTK_CHECK_VERSION(2,12,0)
+static VALUE
+widget_modify_cursor(self, primary, seconday)
+    VALUE self, primary, seconday;
+{
+    gtk_widget_modify_cursor(_SELF(self), 
+                             RVAL2BOXED(primary, GDK_TYPE_COLOR),
+                             RVAL2BOXED(seconday, GDK_TYPE_COLOR));
+    return self;
+}
+#endif
+
 static VALUE
 widget_create_pango_context(self)
     VALUE self;
@@ -685,7 +697,7 @@ static VALUE
 widget_set_double_buffered(self, double_buffered)
     VALUE self, double_buffered;
 {
-    gtk_widget_set_double_buffered(_SELF(self), RTEST(double_buffered));
+    gtk_widget_set_double_buffered(_SELF(self), RVAL2CBOOL(double_buffered));
     return self;
 }
 
@@ -693,7 +705,7 @@ static VALUE
 widget_set_redraw_on_allocate(self, redraw_on_allocate)
     VALUE self, redraw_on_allocate;
 {
-    gtk_widget_set_redraw_on_allocate(_SELF(self), RTEST(redraw_on_allocate));
+    gtk_widget_set_redraw_on_allocate(_SELF(self), RVAL2CBOOL(redraw_on_allocate));
     return self;
 }
 
@@ -709,17 +721,16 @@ static VALUE
 widget_set_scroll_adjustments(self, h, v)
     VALUE self, h, v;
 {
-    gboolean res = gtk_widget_set_scroll_adjustments(_SELF(self),
-                                                     GTK_ADJUSTMENT(_SELF(h)),
-                                                     GTK_ADJUSTMENT(_SELF(v)));
-    return res ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_widget_set_scroll_adjustments(_SELF(self),
+                                                        GTK_ADJUSTMENT(_SELF(h)),
+                                                        GTK_ADJUSTMENT(_SELF(v))));
 }
 
 static VALUE
 widget_mnemonic_activate(self, group_cycling)
     VALUE self, group_cycling;
 {
-    return gtk_widget_mnemonic_activate(_SELF(self), RTEST(group_cycling)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_widget_mnemonic_activate(_SELF(self), RVAL2CBOOL(group_cycling)));
 }
 
 /*
@@ -740,7 +751,7 @@ rc_property_parser(pspec, rc_string, property_value)
     VALUE ret = rb_funcall(func, id_call, 2, spec, CSTR2RVAL(rc_string->str));
     if (NIL_P(ret)) {
         return FALSE;
-    } else if (RTEST(ret)){
+    } else if (RVAL2CBOOL(ret)){
         rbgobj_rvalue_to_gvalue(ret, property_value);
         return TRUE;
     } else {
@@ -762,7 +773,7 @@ widget_s_install_style_property(self, spec)
 
     gclass = (GtkWidgetClass *)g_type_class_ref(cinfo->gtype);
     if (rb_block_given_p()){
-        VALUE func = G_BLOCK_PROC();
+        VALUE func = rb_block_proc();
         rb_hash_aset(style_prop_func_table, spec, func);
         gtk_widget_class_install_style_property_parser(gclass, pspec, 
                                                        (GtkRcPropertyParser)rc_property_parser);
@@ -822,7 +833,7 @@ widget_s_style_properties(argc, argv, self)
 
     ary = rb_ary_new();
     for (i = 0; i < n_properties; i++){
-        if (RTEST(inherited_too)
+        if (RVAL2CBOOL(inherited_too)
             || GTYPE2CLASS(props[i]->owner_type) == self)
             rb_ary_push(ary, CSTR2RVAL(props[i]->name));
     }
@@ -899,9 +910,26 @@ static VALUE
 widget_child_focus(self, direction)
     VALUE self, direction;
 {
-    return gtk_widget_child_focus(_SELF(self), RVAL2GENUM(direction, GTK_TYPE_DIRECTION_TYPE)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_widget_child_focus(_SELF(self), RVAL2GENUM(direction, GTK_TYPE_DIRECTION_TYPE)));
 }
 
+#if GTK_CHECK_VERSION(2,12,0)
+static VALUE
+widget_error_bell(self)
+    VALUE self;
+{
+    gtk_widget_error_bell(_SELF(self));
+    return self;
+}
+
+static VALUE
+widget_keynav_failed(self, direction)
+    VALUE self, direction;
+{
+    return CBOOL2RVAL(gtk_widget_keynav_failed(_SELF(self), RVAL2GENUM(direction, GTK_TYPE_DIRECTION_TYPE)));
+}
+#endif
+
 static VALUE
 widget_child_notify(self, child_property)
     VALUE self, child_property;
@@ -922,7 +950,7 @@ static VALUE
 widget_get_child_visible(self)
     VALUE self;
 {
-    return gtk_widget_get_child_visible(_SELF(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_widget_get_child_visible(_SELF(self)));
 }
 
 static VALUE
@@ -985,7 +1013,7 @@ static VALUE
 widget_set_child_visible(self, is_visible)
     VALUE self, is_visible;
 {
-    gtk_widget_set_child_visible(_SELF(self), RTEST(is_visible));
+    gtk_widget_set_child_visible(_SELF(self), RVAL2CBOOL(is_visible));
     return self;
 }
 
@@ -1010,7 +1038,7 @@ static VALUE
 widget_set_no_show_all(self, no_show_all)
     VALUE self, no_show_all;
 {
-    gtk_widget_set_no_show_all(_SELF(self), RTEST(no_show_all));
+    gtk_widget_set_no_show_all(_SELF(self), RVAL2CBOOL(no_show_all));
     return self;
 }
 
@@ -1045,6 +1073,32 @@ widget_remove_mnemonic_label(self, label)
 }
 #endif
 
+#if GTK_CHECK_VERSION(2,12,0)
+static VALUE
+widget_set_tooltip_window(self, custom_window)
+    VALUE self, custom_window;
+{
+    gtk_widget_set_tooltip_window(_SELF(self), GTK_WINDOW(RVAL2GOBJ(custom_window)));
+    return self;
+}
+
+static VALUE
+widget_get_tooltip_window(self)
+    VALUE self;
+{
+    return GOBJ2RVAL(gtk_widget_get_tooltip_window(_SELF(self)));
+}
+
+static VALUE
+widget_trigger_tooltip_query(self)
+    VALUE self;
+{
+    gtk_widget_trigger_tooltip_query(_SELF(self));
+    return self;
+}
+
+#endif
+
 #if GTK_CHECK_VERSION(2,10,0)
 static VALUE
 widget_get_action(self)
@@ -1065,13 +1119,11 @@ static VALUE
 widget_window(self)
     VALUE self;
 {
-    GdkWindow* window = _SELF(self)->window;
-    return window ? GOBJ2RVAL(window) : Qnil;
+    return GOBJ2RVAL(_SELF(self)->window);
 }
 
 static VALUE
-widget_set_window(self, window)
-    VALUE self;
+widget_set_window(VALUE self, VALUE window)
 {
     _SELF(self)->window = RVAL2GOBJ(window);
     return self;
@@ -1247,6 +1299,9 @@ Init_gtk_widget()
     rb_define_method(gWidget, "modify_text", widget_modify_text, 2);
     rb_define_method(gWidget, "modify_base", widget_modify_base, 2);
     rb_define_method(gWidget, "modify_font", widget_modify_font, 1);
+#if GTK_CHECK_VERSION(2,12,0)
+    rb_define_method(gWidget, "modify_cursor", widget_modify_cursor, 2);
+#endif
     rb_define_method(gWidget, "create_pango_context", widget_create_pango_context, 0);
     rb_define_method(gWidget, "pango_context", widget_get_pango_context, 0);
     rb_define_method(gWidget, "create_pango_layout", widget_create_pango_layout, -1);
@@ -1263,6 +1318,10 @@ Init_gtk_widget()
     rb_define_method(gWidget, "style_get_property", widget_style_get_property, 1);
     rb_define_method(gWidget, "accessible", widget_get_accessible, 0);
     rb_define_method(gWidget, "child_focus", widget_child_focus, 1);
+#if GTK_CHECK_VERSION(2,12,0)
+    rb_define_method(gWidget, "error_bell", widget_error_bell, 0);
+    rb_define_method(gWidget, "keynav_failed", widget_keynav_failed, 0);
+#endif
     rb_define_method(gWidget, "child_notify", widget_child_notify, 1);
     rb_define_method(gWidget, "freeze_child_notify", widget_freeze_child_notify, 0);
     rb_define_method(gWidget, "child_visible?", widget_get_child_visible, 0);
@@ -1285,6 +1344,11 @@ Init_gtk_widget()
     rb_define_method(gWidget, "add_mnemonic_label", widget_add_mnemonic_label, 1);
     rb_define_method(gWidget, "remove_mnemonic_label", widget_remove_mnemonic_label, 1);
 #endif
+#if GTK_CHECK_VERSION(2,12,0)
+    rb_define_method(gWidget, "set_tooltip_window", widget_set_tooltip_window, 1);
+    rb_define_method(gWidget, "tooltip_window", widget_get_tooltip_window, 0);
+    rb_define_method(gWidget, "trigger_tooltip_query", widget_trigger_tooltip_query, 0);
+#endif
 #if GTK_CHECK_VERSION(2,10,0)
     rb_define_method(gWidget, "action", widget_get_action, 0);
     rb_define_method(gWidget, "composited?", widget_is_composited, 0);
