@@ -1,21 +1,26 @@
-require 'gtk2'
-$:.push(File::join(File.dirname(__FILE__),'/src'))
+#!/usr/bin/env ruby
+=begin
+  test.rb - Ruby/GtkSourceView sample script.
+
+  Copyright (c) 2006 Ruby-GNOME2 Project Team
+  This program is licenced under the same licence as Ruby-GNOME2.
+
+  $Id: test.rb,v 1.4 2007/06/03 02:11:07 mutoh Exp $
+=end
 
 require 'gtksourceview'
-def sw(a) 
-	w = Gtk::ScrolledWindow.new
-	w.add(a)
-	w
-end
 
 w = Gtk::Window.new
 w.signal_connect("delete-event"){Gtk::main_quit}
-w.add(sw(b = Gtk::SourceView.new))
-b.show_line_numbers = 1
-#p b.buffer.methods
+
+view = Gtk::SourceView.new
+w.add(Gtk::ScrolledWindow.new.add(view))
+view.show_line_numbers = true
+
 lang = Gtk::SourceLanguagesManager.new.get_language('text/x-ruby')
-b.buffer.language = lang
-b.buffer.highlight = true
+view.buffer.language = lang
+view.buffer.highlight = true
+
 w.set_default_size(400,300)
 # customize style
 lang.tags.each do |tag|
