@@ -1,6 +1,6 @@
 #
 # pango.rb
-# Copyright(C) 2005 Ruby-GNOME2 Project.
+# Copyright(C) 2005, 2009 Ruby-GNOME2 Project.
 #
 # This program is licenced under the same
 # license of Ruby-GNOME2.
@@ -16,11 +16,6 @@ require 'pango.so'
 module Pango
   LOG_DOMAIN = "Pango"
 
-  def cairo_available?
-    Pango.constants.include?("CairoFontMap")
-  end
-  module_function :cairo_available?
-
   class AttrList
     def each(text = nil)
       iter = iterator
@@ -31,7 +26,7 @@ module Pango
         else
           yield(iter)
         end
-     end while iter.next!
+      end while iter.next!
     end
   end
 
@@ -51,7 +46,7 @@ module Pango
   targets.each do |klass, prefix|
     (klass.constants - klass.superclass.constants).each do |name|
       unless klass.const_get(name).is_a? Class
-	const_set(prefix + name, klass.const_get(name))
+	const_set("#{prefix}#{name}", klass.const_get(name))
       end
     end
   end
