@@ -13,24 +13,16 @@ $LOAD_PATH.unshift MKMF_GNOME2_DIR
 
 require 'mkmf-gnome2'
 
-PKGConfig.have_package(PACKAGE_ID, 0, 5, 2) or exit 1
+PKGConfig.have_package(PACKAGE_ID, 0, 8, 0) or exit 1
 setup_win32(PACKAGE_NAME)
 
-if PKGConfig.have_package('cairo') and have_header('rb_cairo.h')
-  if /mingw|cygwin|mswin32/ =~ RUBY_PLATFORM
-    unless ENV["CAIRO_PATH"]
-      puts "Error! Set CAIRO_PATH."
-      exit 1
-    end
-    add_depend_package("cairo", "packages/cairo/ext", ENV["CAIRO_PATH"])
-    $defs << "-DRUBY_CAIRO_PLATFORM_WIN32"
-  end
-end
+check_cairo
 
 add_depend_package("glib2", "glib/src", TOPDIR)
+add_depend_package("gtk2", "gtk/src", TOPDIR)
+add_depend_package("gdk_pixbuf2", "gdkpixbuf", TOPDIR)
 
 make_version_header("POPPLER", PACKAGE_ID)
-have_func("poppler_page_render_selection_to_pixbuf")
 
 create_makefile_at_srcdir(PACKAGE_NAME, SRCDIR, "-DRUBY_POPPLER_COMPILATION")
 create_top_makefile
